/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.DrawingTools;

import java.awt.*;
import javax.swing.*;

/**
 * BrushTool - Drawing tool for simple strokes.  Strokes more closely
 * 	resemble pencil strokes.  <b>Most functionality has been moved to
 *  the BackgroundSurface mouselisteners.</b>
 * 
 * @author Terry Bradley
 * @version 04.7.29
 *
 */
public class BrushTool extends Tool {
	
	public static int THRESHOLD = 12;		// Only repaint drawingSurface
											// if line dist < THRESHOLD
	
	public static int STROKE_THICKNESS = 1;	// thickness of stroke in px
	public static BasicStroke STROKE;		// stroke instance
	
//////////////////////////  CONSTRUCTORS  /////////////////////////////////////
	/**
	 * Constructor - instance of Brush tool with default thickness.
	 */
	public BrushTool() {
		super();		
		init();
	}
	
	/**
	 * Constructor - instance of Brush tool with given cursor,
	 * name, icon, and thickness.
	 * 
	 * @param c Brush cursor
	 * @param s Name ??
	 * @param i Icon for toolbox
	 * @param thick Thickness in px
	 */
	public BrushTool(Cursor c, String s, ImageIcon i, int thick)
	{
		super(c,s,i);
		STROKE_THICKNESS = thick;		
		init();
	}
	
	/**
	 * Constructor helper method initializes STROKE
	 */
	private void init(){
		STROKE = new BasicStroke(STROKE_THICKNESS);
	}
	
//////////////////////////  METHODS  /////////////////////////////////////
	
	/**
	 * Sets the thickness and updates STROKE.
	 * 
	 * @param f Thickness in px
	 */
	public static void setThickness(int f) { 
		if( STROKE_THICKNESS == f )
			return;
		STROKE_THICKNESS = f;
		STROKE = new BasicStroke(STROKE_THICKNESS); 
	}
}