/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.DrawingTools;

import java.awt.*;
import javax.swing.*;

/**
 * ErasorTool - Drawing tool for erasing.  <b>Most 
 *  functionality has been moved to the BackgroundSurface 
 *  mouselisteners.</b>
 * 
 * @author Terry Bradley
 * @version 04.7.29
 *
 */
public class ErasorTool extends Tool {
	
	protected static int THRESHOLD = 10;
	
	protected static int ERASE_SIZE = 3;
	protected static BasicStroke STROKE;

	/**
	 * Constructor - Default erasor tool
	 *
	 */
	public ErasorTool() {
		super();
		STROKE = new BasicStroke(ERASE_SIZE);
	}
	
	/**
	 * Constructor - Erasor with given cursor, name, icon, and eraser size.
	 * 
	 * @param c Cursor
	 * @param s Name
	 * @param i Icon for toolbox
	 * @param thick Eraser size (thick x thick pixel square)
	 */
	public ErasorTool(Cursor c, String s, ImageIcon i, int thick)
	{
		super(c,s,i);
		STROKE = new BasicStroke(thick);
		ERASE_SIZE = thick;
	}
	
	/**
	 * Sets the size of the erase area to f x f px.
	 * 
	 * @param f Length of side of square defining erase area.
	 */
	public static void setThickness(int f) { 
		if( ERASE_SIZE == f )
			return;
		ERASE_SIZE = f;
		STROKE = new BasicStroke(ERASE_SIZE); 
	}
}