/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.ActionGrid;

import java.awt.*;
import javax.swing.table.*;
import javax.swing.*;

import edu.uiuc.orchid.clover.Data.*;
import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Actions.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;

/**
 * Table renderer takes a table value, in Long or Integer form, and
 * displays the appropriate ActionButton.
 * 
 * @author Terry J Bradley
 * @version 04.7.31
 *
 */
public class DialogTableRenderer extends JPanel implements TableCellRenderer {
	
	protected Scene m_scene;
	protected boolean m_isColHeader;
	protected boolean m_isGray = false;
	
	static public ImageIcon SPEAK_ICON;
	static public ImageIcon THINK_ICON;
	static public ImageIcon EMOTE_ICON;
	
	/**
	 * Constructor - Requires a link to the scene.  Headers
	 * correspond to renderers for the first column.
	 * @param sty
	 * @param isHeader
	 */
	public DialogTableRenderer(Scene sty, boolean isHeader) {
		super();
		m_scene = sty;
		m_isColHeader = isHeader;
	}
	
	/**
	 * Required by TableCellRenderer.  Sets up this JPanel to display
	 * action.
	 */
	public Component getTableCellRendererComponent(JTable table, Object value,
		boolean isSelected, boolean hasFocus, int nRow, int nCol)
	{
		// Do most work here:
		setValue(value, nRow, nCol);
		
		setBackground(
			( isSelected && m_isColHeader ) ? 
				table.getSelectionBackground() : table.getBackground());
		setForeground(isSelected && (m_isColHeader || hasFocus) ? 
			table.getSelectionForeground() : table.getForeground());
		setFont(table.getFont());
		// TODO: (If Needed) set focus border / no focus border
		
		// Adjust height
		int width = table.getColumnModel().getColumn(nCol).getWidth();
		setSize(width, 1000);
		int rowHeight = Constants.ROW_HEIGHT;
		if( table.getRowHeight(nRow) != rowHeight )
			table.setRowHeight(nRow, rowHeight);
			
		return this;
	}
	
	/**
	 * Display helper does most work in setting up visual depiction
	 * of action.
	 * 
	 * @param value Value of cell, should be Long or Integer
	 * @param row Row
	 * @param col Column
	 */
	public void setValue(Object value, int row, int col) {
		// Need to recognize: String(""), Integer, Long
		if( value instanceof Long ) {
			removeAll();
			
			// Look up gid in story.getImageFromGID; .getNameFromGID
			int gid = ((Long)value).intValue();
			JLabel lbl;
			if( gid < 0 )
				lbl = new JLabel("");
			else
				lbl = new JLabel(m_scene.getCharacter(new Integer(gid)).getName(),
								m_scene.getCharacter(new Integer(gid)).getThumbnail(),
								JLabel.CENTER);
			lbl.setHorizontalTextPosition(JLabel.CENTER);
			lbl.setVerticalTextPosition(JLabel.BOTTOM);
			add(lbl);
		}
		else if( value instanceof Integer ) {
			
			removeAll();
			setLayout(new GridBagLayout());
			GridBagConstraints c = new GridBagConstraints();
			c.weightx = 0.0;
			
			// Test for Speak action
			boolean empty = true;
			PrimaryAction ca = null;
			String text = "<html>";
			int val = ((Integer)value).intValue();
			
			Long gid = m_scene.getCharacterIDAtPosition(row);
			
			ca = m_scene.getPrimaryAction(gid, col);			
			c.anchor = GridBagConstraints.SOUTHEAST;
			c.weighty = 0.3;
			c.insets = new Insets(0, 32, 0, 32);
			if( ( val & Constants.SPEAK_OFFSET ) == Constants.SPEAK_OFFSET ) {
				// Need to lookup action somehow
				JLabel lbl = new JLabel(SPEAK_ICON);
				lbl.setToolTipText("Speak Action: Right-click to edit.");
				lbl.setEnabled(!m_isGray);
				add(lbl, c);
				empty = false;
			}
			// Test for Think action
			if( ( val & Constants.THINK_OFFSET ) == Constants.THINK_OFFSET ) {
				JLabel thk = new JLabel(THINK_ICON);
				thk.setToolTipText("Think Action: Right-click to edit.");
				thk.setEnabled(!m_isGray);
				add(thk, c);
				empty = false;
			}
			text = null;
			if(ca != null)
				text = CloverWindow.formatText(ca.getActionText(), 20, 3);
			
			boolean addSec = false;

			// Test for Emotion
			Emote emote = (Emote)m_scene.getSecondaryAction(gid, col);
				
			JLabel emo = new JLabel();
			if(ca != null)
				emo.setIcon(Emote.getEmoticon(emote.m_emotion).m_image);
			emo.setToolTipText("Emotion: Replace using the Emoticon Toolbox.");
			emo.setEnabled(!m_isGray);
			c.gridy = 1;
			c.anchor = GridBagConstraints.NORTHWEST;
			add(emo, c);
			
			addSec = true;
					
			/**
			 * Old code checked for TTS audio or recorded audio and displayed
			 * a Speaker icon if action had either.  This would remind
			 * author which actions needed audio recordings.  Currently
			 * disabled.
			 */
			/*
			if( ( val & Constants.HAS_AUDIO_OFFSET ) == Constants.HAS_AUDIO_OFFSET ){
				JLabel thk = new JLabel(Clover.createImageIcon("JavaIcons/Volume24.gif"));
				thk.setToolTipText("Character Audio: Click and hold to preview.");
				secondary.add(thk);
				addSec = true;
			}
			*/
			//if(addSec)
			//	add(secondary);
			
			// Add text
			c.gridx = 0;
			c.gridy = 2;
			c.gridheight = 1;
			c.weighty = .7;
			c.insets = new Insets(3,3,3,3);
			c.anchor = GridBagConstraints.CENTER;
			if (text != null) {
				JLabel txt = new JLabel(text);
				txt.setForeground( m_isGray ? Color.gray : Color.black);
				add(txt, c); 
			}
		}
		else { // String, i.e. "" or " " for blank cols
			System.err.println("Trying to render a non-Int, non-Long");
			if( value instanceof String ) {
				JLabel lbl = new JLabel((String)value + " ");
				add(lbl);
			}
		}
	}
	
	/**
	 * Grays or colors icon.
	 */
	public void setEnabled(boolean b){
		m_isGray = !b;
		super.setEnabled(b);
	}
	
	/**
	 * Statically load the default image icons.  EMOTE_ICON may
	 * not be at all useful.
	 *
	 */
	
	public static void loadImages() {
		SPEAK_ICON = Clover.createImageIcon2(Clover.SPEAK_ICON_LOC);
		THINK_ICON = Clover.createImageIcon2(Clover.THINK_ICON_LOC);
		EMOTE_ICON = Clover.createImageIcon2(Clover.EMOTE_ICON_LOC);
	}
			
}