/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface;

import java.io.File;
import java.io.FileFilter;

/**
 * Provides a filter for *.clover files.  Based on sample code in
 * Swing Second Edition (manning)
 * 
 * @author Terry J Bradley
 * @version 04.7.30
 *
 */
public class CloverFilter extends javax.swing.filechooser.FileFilter 
						  implements FileFilter 
{
	private String description = null;	// Description of file type
	private String extension = null;	// Extension of file type
	private boolean navigation = true;	// True allows directory navigation
	
	/**
	 * Creates a file filter of arbitrary extention and description. By
	 * default, navigation of other directories is allowed.
	 *  
	 * @param ext Extension
	 * @param des Description
	 */
	public CloverFilter(String ext, String des){
		extension = ext;
		description = des;
	}
	
	/**
	 * Creates a Clover file filter.  ext = *.clover, description =
	 * "Clover Story Files (*.clover)".
	 * 
	 * @param nav Sets directory navigation status
	 */
	public CloverFilter(boolean nav){
		this(".clover", "Clover Story Files (*.clover)");
		navigation = nav;
	}
	
	
	/**
	 * Gets the filter description
	 * 
	 * @return Description
	 */
	public String getDescription(){
		return description;
	}
	
	/**
	 * Checks file to see whether it should be displayed by filter.
	 * Allows directories iff navigation is set, and allows files
	 * with .endsWith(ext).
	 * 
	 * @param f File to check
	 * @return True iff file is accepted by filter.
	 */
	public boolean accept(File f){
		if (f==null)
			return false;
		if (f.isDirectory())
			return navigation;
		return f.getName().toLowerCase().endsWith(extension);
	}
}