/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface;

import javax.swing.*;
import java.awt.*;

/**
 * DragGlassPane implements a simple glassPane for use with
 * custom drag and drop functionality.
 * 
 * @author Terry J Bradley
 * @version 04.7.30
 *
 */
public class DragGlassPane extends JComponent {
	Point point;		// Storage point
	Rectangle rect;		// Storage rectangle

	/**
	 * Constructor - creates a simple JComponent.
	 *
	 */
	public DragGlassPane(){
		setLayout(null);
		setVisible(true);
		point = new Point(0,0);
		rect = null;
	}
	
	
	/**
	 * Overwritten paintComponent draws a red rectangle.  This
	 * functionality is used in conjunction with Scene Preview
	 * in the Scene Editor.
	 */
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);
		if(rect != null){
			g.setColor(Color.red);
			g.drawRect(rect.x, rect.y, rect.width, rect.height);
		}
	}

	/**
	 * Sets the storage point
	 * 
	 * @param p Point
	 */
	public void setPoint(Point p) {
		point = p;
	}
	
	/**
	 * Sets the playback rectangle
	 * 
	 * @param r Rectangle
	 */
	public void setRectangle(Rectangle r){
		rect = r;
	}

}