/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Tabs;

import javax.swing.*;
import javax.swing.border.EmptyBorder;

import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.Data.Story;
import edu.uiuc.orchid.clover.DrawingTools.DrawingSurface;

/**
 * CloverSplitPane
 * Simple superclass for ViewAndEdit, CharacterUI, and later
 * ResponseUI.
 * 
 * @author Terry Bradley
 * @version 04.8.11
 *
 */
public class CloverSplitPane extends JSplitPane{
	
	protected CloverWindow m_parent;	// Parent window
	protected Thread th_updateThumb;	// Thread for updating thumbnails,
										// overwritten by extensions
	protected javax.swing.Timer m_timer;// Timer for updating thumbnails,
										// allows multiple requests to
										// coalesce
	
	/**
	 * Constructor - Only requires the parent window.
	 * 
	 * @param frame Parent CloverWindow
	 */
	public CloverSplitPane(CloverWindow frame){
		super();
		m_parent = frame;
		setBorder(new EmptyBorder(3,3,3,3));
	}
	
	/**
	 * Returns parent window
	 * 
	 * @return Parent window
	 */
	public CloverWindow getWindow(){
		return m_parent;
	}
	
	/**
	 * Returns drawing surface.  Typically overwritten
	 * by extensions to do something useful.
	 * 
	 * @return Drawing surface
	 */
	public DrawingSurface getDrawingSurface(){
		return null;
	}
	
	/**
	 * Starts the update timer, or resets it if already running.
	 * This allows multiple requests to coalesce.  Once the requests
	 * "quiet down", the timer expires and runs the updateThumb
	 * thread.
	 * 
	 * @param id Scene or character whose thumbnail needs updating.
	 */
	public void updateThumbnail(Integer id){
		if(m_timer.isRunning())
			m_timer.stop();
		m_timer.start();
	}
	
	/**
	 * Returns the main story instance.
	 * 
	 * @return Story
	 */
	public Story getStory(){
		return m_parent.getStory();
	}
	
	/**
	 * Gets the previous item in the list.  This function
	 * should be overwritten by extension classes.
	 * 
	 * @param base Item whose predecesor this function searches for.
	 * @return Previous item
	 */
	public Integer getPreviousItemInList(Integer base){
		return new Integer(-1);
	}
	
	/**
	 * Stops the update thumbnail timer without restarting it.
	 *
	 */
	public void stopTimer(){
		m_timer.stop();
	}
	
	/**
	 * Empty function for updating a gallery, to be overwritten
	 * by extensions.
	 * 
	 * @param responseIndex ??
	 */
	public void updateGallery(int responseIndex){}
	
}