/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Tabs;

import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.Interface.*;

/**
 * FileUI - User interface for saving, opening, and creating
 * new stories.
 * 
 * @author Terry J Bradley
 * @version 04.8.12
 *
 */
public class FileUI extends JPanel implements Constants {
	
	protected CloverWindow m_parent;		// Parent window
	protected JLabel m_save;				// Save status label
	protected JButton jb_save;				// Save button
	protected JButton jb_saveAs;			// SaveAs button
	protected JRadioButton jrb_newStory;	// New Story button
	protected JRadioButton jrb_openStory;	// Open Story button
	protected JTextField jtf_newStory;		// New Story text box
	public PreviewBox pb_preview;			// Thumbnail story preview
	protected JButton jb_browse;			// Browse button used for
											// browsing for file OR
											// opening a file in the list
	protected JList m_list;					// List of existing stories
	protected DefaultListModel m_listModel;	// Above's list model
	
	private int m_previewIndex = -1;		// Currently displayed
											// thumbnail preview
	
	public final static int STATUS_SAVED = 0;	// Save status values
	public final static int STATUS_FAILED = 1;
	public final static int STATUS_BLANK = 2;
	
	/**
	 * Constructor - Creates the File interface and adds all necessary
	 * components.  Many components are globally declared to enable
	 * dynamic enabling/disabling, updated contents, etc.
	 * 
	 * @param win CloverWindow parent
	 */
	public FileUI(CloverWindow win) {
		super();
		m_parent = win;
		
		// Contents: titleBorder containing gridbaglayout
		JPanel inner = new JPanel();
		inner.setBorder(
			new CompoundBorder(
				new LineBorder(Color.black),
				new EmptyBorder(10,20,10,20)) );
		inner.setLayout(new GridBagLayout());
				
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(2,2,2,2);
		c.anchor = GridBagConstraints.WEST;
				
		// Save labels/buttons
		ImageIcon img = Clover.createImageIcon2(Clover.ICON_DIR+"Save24.gif");
		m_save = new JLabel("Save Story:  \"" + m_parent.getStoryName()
			+ "\"", img, SwingConstants.LEFT);
		m_save.setBorder(new EmptyBorder(0,0,5,0));
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 3;
		c.weightx = 1.0;
		inner.add(m_save,c);			
			
		jb_save = new JButton("Save Changes");
		jb_save.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e){
				setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
				// Attempt to save AND update label to reflect
				// success/failure.
				setSaveLabel(m_parent.saveStory());
				setCursor(Cursor.getDefaultCursor());
			}
		});		
		
		jb_saveAs = new JButton("Save As...");
		jb_saveAs.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e ) {
				// Attempt to save AND update label to reflect
				// success/failure.
				setSaveLabel(m_parent.saveStoryAs());
			}
		});

		JPanel p = new JPanel(new FlowLayout(FlowLayout.TRAILING,0,0));
		p.setBorder(new EmptyBorder(0,0,10,0));
		p.add(jb_save);
		p.add(Box.createHorizontalStrut(5));
		p.add(jb_saveAs);
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 3;
		c.anchor = GridBagConstraints.EAST;
		inner.add(p, c);		
				
		// Divider
		JLabel divider = new JLabel("");
		divider.setBorder(new CompoundBorder(
				new EmptyBorder(0, 0, 10, 0),
				new MatteBorder(1, 50, 1, 50, Color.black)));
		c.gridy = 2;
		c.anchor = GridBagConstraints.CENTER;
		c.fill = GridBagConstraints.HORIZONTAL;
		inner.add(divider, c);
		
		// New story radio button and stuff
		img = Clover.createImageIcon2(Clover.ICON_DIR+"New24.gif");
		jrb_newStory = new JRadioButton("Create a new story entitled: ");
		jrb_newStory.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				jtf_newStory.selectAll();
				jb_browse.setEnabled(jtf_newStory.getText().length() > 0);
				jtf_newStory.requestFocus();
			}
		});
		ButtonGroup radios = new ButtonGroup();
		radios.add(jrb_newStory);
		
		jtf_newStory = new JTextField(20);
		jtf_newStory.addKeyListener(new KeyAdapter(){
			public void keyReleased(KeyEvent e){
				jrb_newStory.setSelected(true);
				jb_browse.setEnabled((jtf_newStory.getText().length() > 0));
			}
		});
		jtf_newStory.addFocusListener(new FocusAdapter() {
			public void focusGained(FocusEvent e) {
				super.focusGained(e);
				jrb_newStory.setSelected(true);
				jb_browse.setEnabled(jtf_newStory.getText().length() > 0);
			}
		});
		jtf_newStory.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				if(jtf_newStory.getText().length() > 0)
					jb_browse.doClick();
			}
		});
		
		JPanel nwStory = new JPanel(new FlowLayout(FlowLayout.LEFT,0,0));
		nwStory.add(jrb_newStory);
		nwStory.add(jtf_newStory);
		c.gridx = 0;
		c.gridy = 3;
		c.gridwidth = 3;
		c.fill = GridBagConstraints.NONE;
		c.anchor = GridBagConstraints.WEST;
		inner.add(nwStory, c);
		
		// Open labels/buttos
		img = Clover.createImageIcon2(Clover.ICON_DIR+"Open24.gif");
		jrb_openStory = new JRadioButton("Open an exisiting story:");	
		jrb_openStory.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e) {
				m_list.requestFocus();
				jb_browse.setEnabled(!m_list.isSelectionEmpty());
			}
		});
		c.gridx = 0;
		c.gridy = 4;
		c.gridwidth = 3;
		c.anchor = GridBagConstraints.WEST;
		inner.add(jrb_openStory, c);
		radios.add(jrb_openStory);
		
		// List of prev stories plus preview	
		c.gridy = 5;
		c.gridx = 0;
		c.gridwidth = 1;
		inner.add(Box.createHorizontalStrut(10),c);
		
		m_listModel = new DefaultListModel();
		m_list = new JList(m_listModel);
		m_list.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		m_list.setBorder(new LineBorder(Color.black));
		m_list.addFocusListener(new FocusListener(){
			public void focusGained(FocusEvent e){
				jrb_openStory.setSelected(true);
				jb_browse.setEnabled(true);
				pb_preview.setEnabled(true);
			}
			public void focusLost(FocusEvent e){
				// Disable preview image
				pb_preview.setEnabled(false);
			}
		});
		m_list.addListSelectionListener(new ListSelectionListener(){
			public void valueChanged(ListSelectionEvent e){
				// Update preview image
				if(!(m_list.getSelectedValue() instanceof FileListObject)) {
					pb_preview.setIcon(null);
					m_previewIndex = -1;
					return;
				}
				if(m_previewIndex == m_list.getSelectedIndex())
					return;
				m_previewIndex = m_list.getSelectedIndex();
				m_parent.previewStory(
						((FileListObject)m_list.getSelectedValue()).path);
			}
		});
		m_list.addMouseListener(new MouseAdapter(){
			public void mouseClicked(MouseEvent e){
				if(e.getClickCount() > 1)
					jb_browse.doClick();
			}
		});
		c.anchor = GridBagConstraints.WEST;
		c.gridx = 1;
		c.gridy = 5;
		c.gridwidth = 1;
		m_list.setPreferredSize(new Dimension(200,100));
		inner.add(m_list,c);
		
		pb_preview = new PreviewBox();
		c.anchor = GridBagConstraints.EAST;
		c.gridx = 2;
		c.gridwidth = 1;
		c.fill = GridBagConstraints.NONE;
		inner.add(pb_preview, c);
		
		// Updated Go button, either opens a file chooser
		// or opens the selected story
		jb_browse = new JButton("Ok", img);
		jb_browse.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				// TODO: Current save checking is very stupid, and
				// can be improved.
				if(m_parent.hasStory()){
					int choice = JOptionPane.showConfirmDialog(
						FileUI.this, "Save changes to current story, \"" 
						+ m_parent.getStory().getTitle() + "?\"", 
						"Save changes?", 
						JOptionPane.YES_NO_CANCEL_OPTION);
					switch(choice){
						case JOptionPane.CANCEL_OPTION:
							return;
						case JOptionPane.YES_OPTION:
							jb_save.doClick();
							break;
					}
				}
				
				if(jrb_newStory.isSelected()){
					if(jtf_newStory.getText().length() > 0)
						m_parent.createNewStory(jtf_newStory.getText());
					else
						m_parent.createNewStory();
				}
				else if(jrb_openStory.isSelected()){				
					Object obj = m_listModel.getElementAt(m_list.getSelectedIndex());
					if(obj instanceof SpecialFileListObject){
						if(((SpecialFileListObject)obj).operation == 0)
							m_parent.createNewStory();
						else {
							int result = m_parent.openStory();
							if(result == STATUS_FAILED)
								JOptionPane.showMessageDialog(
											m_parent, 
											"Error Opening File.  File is likely corrupt.",
											"Open File Error",
											JOptionPane.ERROR_MESSAGE);
						}
					}
					if(! (obj instanceof FileListObject))
						return;
					m_parent.openStory(new File(((FileListObject)obj).path));
				}
			}
		});
		
		// New = Create new story, prompt for Name, and move to Narrative.
		p = new JPanel(new FlowLayout(FlowLayout.TRAILING,0,0));
		p.add(jb_browse);
		c.gridx = 0;
		c.gridy = 6;
		c.gridwidth = 3;
		c.anchor = GridBagConstraints.EAST;
		inner.add(p, c);
		
		
		JPanel pane = new JPanel();
		pane.setLayout(new GridBagLayout());
		c = new GridBagConstraints();
		c.anchor = GridBagConstraints.CENTER; 
		pane.setBackground(Constants.COLOR_BACKDROP);
		pane.setBorder(new EmptyBorder(10,10,10,10));
		pane.add(inner, c);	

		setBorder(new EmptyBorder(3,3,3,3));
		setLayout(new BorderLayout());
		add(pane, BorderLayout.CENTER);
		
		updateFileUI();
		openTab();
	}
	
	/**
	 * Highlights the text in the Story Name text box.
	 *
	 */
	public void highlightNameTextfield(){
		jtf_newStory.requestFocus();
		jtf_newStory.selectAll();
	}
	
	/**
	 * Sets the save status label based on a status code.
	 * 
	 * @param s Story title
	 * @param status Status code
	 */
	public void setSaveLabel(String s, int status) {
		String result = "";
		switch(status){
			case STATUS_SAVED:
				result = "- <b>Save Successful</b>";
			break;
			case STATUS_FAILED:
				result = "- <b>Save Failed</b>";
			break;
		}
		m_save.setText("<html>Save Story:  \""  + s+  "\" " + result + "</html>" );
	}
	
	/**
	 * Sets the save status label based on a status code.
	 * 
	 * @param status Status code
	 */
	public void setSaveLabel(int status){
		setSaveLabel(m_parent.getStoryName(), status);
	}
	
	/**
	 * Updates the preview image based on the list's selected index.
	 * The "Browse..." list item sets a null image, which in fact
	 * loads the "No Preview..." image.
	 *
	 */
	public void updatePreviewImage(){
		if(!(m_list.getSelectedValue() instanceof FileListObject)) {
			pb_preview.setIcon(null);
			m_previewIndex = -1;
			return;
		}
		if(m_previewIndex == m_list.getSelectedIndex())
			return;
		m_previewIndex = m_list.getSelectedIndex();
		m_parent.previewStory(
				((FileListObject)m_list.getSelectedValue()).path);
	}
	
	/**
	 * Updates the fileUI to reflect whether saving is possible,
	 * and to setup the list of existing files.
	 *
	 */
	public void updateFileUI() {
		// Two possible states: Story loaded, and story not loaded.
		if( m_parent.hasStory() ) {
			// Story is loaded - Set label text accordingly
			setSaveLabel(m_parent.getStoryName(), STATUS_BLANK);
			jb_save.setEnabled(true);
			jb_saveAs.setEnabled(true);
		}
		else {
			// No story - disable some buttons
			m_save.setText("Save story:");
			jb_save.setEnabled(false);
			jb_saveAs.setEnabled(false);
		}
		// Now setup the Previous files list.
		updateList2();
	}
	
	/**
	 * Sets up the list of existing files by scanning the Clover.STORY_DIR
	 * directory.  Subdirectories are NOT examined.
	 *
	 */
	public void updateList2(){
		m_listModel.clear();
		m_listModel.addElement(new SpecialFileListObject(1));
		
		File dir = new File(Clover.STORY_DIR);
		if(!dir.isDirectory())
			return;
		
		File[] files = dir.listFiles(new CloverFilter(false));
		for(int i=0; i<files.length; i++){
			if(files[i].isFile()){
				String name = files[i].getName().substring(0, 
						files[i].getName().lastIndexOf('.'));
				FileListObject flo = new FileListObject(
						files[i].getAbsolutePath(), name);
				m_listModel.addElement(flo);
			}
		}
	}
	
	/**
	 * Called when a story is opened.  Same as calling updateFileUI()
	 *
	 */
	public void openStory(){
		updateFileUI();
	}
	
	/**
	 * Called when the user opens the fileUI tab.  Sets text boxes
	 * and radio buttons.
	 *
	 */
	public void openTab(){
		updateFileUI();
		jtf_newStory.setText("");
		
		if(jrb_newStory.isSelected()){
			pb_preview.setEnabled(false);
			jb_browse.setEnabled(false);
		}
		else {
			updatePreviewImage();
			pb_preview.setEnabled(true);
			jb_browse.setEnabled(m_list.getSelectedIndex()>-1);
		}
		
	}	
	
}


/**
 * FileListObject - Existing stories appear in the list as FileListObjects.
 * The storyName is displayed, while the path is used to actually load the
 * story, should the user desire.
 * 
 * @author Terry J Bradley
 * @version 04.8.12
 *
 */
class FileListObject extends Object {
	String path;		// to load story (includes filename)
	String storyName;	// to display story in list
	
	/**
	 * Constructor - Creates a list object with the given path
	 * and story name.
	 * 
	 * @param p Path to story
	 * @param s Story name
	 */
	public FileListObject(String p, String s){
		path = p;
		storyName = s;
	}
	
	/**
	 * Overwritten function simplifies usage with a JList - the
	 * Story name is automatically displayed for this datatype.
	 */
	public String toString(){
		return storyName;
	}
}

/**
 * SpecialFileListObject allows "special" objects to be added to the
 * list.  This class supports a "Create New Story" type (not currently 
 * used) and a "More Stories..." type, which allows the user to browse
 * the file system.
 * 
 * @author Terry J Bradley
 * @version 04.8.12
 *
 */
class SpecialFileListObject {
	int operation;	// 0 = Create New, 1 = Browse...
	private static final String CREATE_NEW = "<Create New Story>";
	private static final String OPEN_STORY = "More stories...";
	
	/**
	 * The SpecialFileListObject consists only of an integer.
	 * This value is set in the constructor.
	 * 
	 * @param op Operation - 0 for create story, 1 for browse file system
	 */
	public SpecialFileListObject(int op){
		operation = op;
	}
	
	/**
	 * Overwritten function simplifies usage with a JList - the
	 * appropriate label is taken from the Strings shown above.
	 */
	public String toString(){
		return (operation == 0) ? CREATE_NEW : OPEN_STORY;
	}
}
