/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Tabs;

import javax.swing.*;
import javax.swing.border.CompoundBorder;
import javax.swing.border.EmptyBorder;
import javax.swing.border.LineBorder;
import javax.swing.border.MatteBorder;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Data.GlobalCharacter;
import edu.uiuc.orchid.clover.Interface.TrashLabel;
import edu.uiuc.orchid.clover.Interface.CloverList.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.Interface.ScriptPane.*;

/**
 * ScriptUI - User interface for writing and editing a vignette script.
 * The script interface is contained primarily in a ScriptPane, which
 * holds and arranges a dynamic number of script elements, such as dialog.
 * This interface also has a CloverList, which appears as the left
 * side of the splitpane.  The list shows all the Story's characters.
 * This list is editable both here and in the Character UI.
 * 
 * ScriptUI interface also contains a comment layer, like the Narrative UI.
 * 
 * @author Terry J Bradley
 * @version 04.8.16
 *
 */
public class ScriptUI extends CloverSplitPane {
	
	public CloverList m_gallery;			// Gallery of character names
	protected ScriptPane jtp_Script;		// ScriptPane containing all script
											// elements.
	protected JPanel rightSide;				// Container for ScriptPane
	protected JButton m_clearComments;		// Button for clearing the comment
											// layer.
	protected JToggleButton m_commentMode;	// Button for setting comment mode
	protected JToggleButton m_showComments;	// Button for showing/hiding comments
	
	static final int OFFSET = 73;			// Offset for margins in ScriptPane
	
	/**
	 * Constructor - Creates a Script Interface.  Creates the character
	 * name gallery for the left side and an empty ScripPane for the
	 * right side.
	 * 
	 * @param owner CloverWindow parent
	 */
	public ScriptUI(CloverWindow owner){
		super(owner);
		
		// Create character gallery and define its button's functionality.
		m_gallery = new CloverList(owner, true, "Add Character");
		m_gallery.setAddButtonAction(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				m_parent.addCharacter();
			}
		});
		
		JPanel leftSide = new JPanel(new BorderLayout());
		leftSide.add(m_gallery, BorderLayout.CENTER);
		
		// Add trash label to left side
		JPanel trashPane = new JPanel();
		trashPane.setLayout(new BoxLayout(trashPane, BoxLayout.Y_AXIS));
		trashPane.add(Box.createVerticalStrut(5));
		trashPane.add(new TrashLabel(this));
		leftSide.add(trashPane, BorderLayout.SOUTH);
		
		setLeftComponent(leftSide);
		
		// Create menu pane.  Buttons in this pane will add new script
		// elements.  Comment layer functionality is also found here.
		JPanel menuPane = new JPanel();
		ImageIcon img = Clover.createImageIcon2(
				Clover.ICON_DIR+"script_icon_24.gif");
		JButton insertLine = new JButton(img);
		insertLine.setToolTipText("Insert new dialog");
		insertLine.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent e){
				if(jtp_Script.getCharacterCount() > 0)
					jtp_Script.addNewDialog(guessNextSpeaker());
			}
		});
		
		img = Clover.createImageIcon2(
				Clover.ICON_DIR+"clear_comments_2-24.gif");
		m_clearComments = new JButton(img);
		m_clearComments.setToolTipText("Clear Comments");
		m_clearComments.setEnabled(false);
		m_clearComments.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				if ( m_parent.confirmDeletion("Remove all comments?",
						"Comments") )
					jtp_Script.getCommentLayer().resetImage();
			}
		});	
		
		img = Clover.createImageIcon2(
				Clover.ICON_DIR+"comment_mode_2-24.gif");
		m_commentMode = new JToggleButton(img);
		m_commentMode.setToolTipText("Add Comments");
		m_commentMode.setSelected(false);
		m_commentMode.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JToggleButton jtb = (JToggleButton)e.getSource();
				jtp_Script.getCommentLayer().setCommentMode(jtb.isSelected());
				if(jtb.isSelected() && !m_showComments.isSelected()) {
					m_showComments.doClick();
				}
				jtp_Script.repaint();
			}
		});	
		
		img = Clover.createImageIcon2(
				Clover.ICON_DIR+"show_hide_comments_2-24.gif");
		m_showComments = new JToggleButton(img);
		m_showComments.setToolTipText("Show Comments");
		m_showComments.setSelected(false);
		m_showComments.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				JToggleButton jtb = (JToggleButton)e.getSource();
				jtp_Script.getCommentLayer().setCommentVisible(jtb.isSelected());
				m_clearComments.setEnabled(jtb.isSelected());
				if(!jtb.isSelected()) {
					jtp_Script.getCommentLayer().setCommentMode(false);
					m_commentMode.setSelected(false);
					m_showComments.setToolTipText("Show Comments");
				}
				else
					m_showComments.setToolTipText("Hide Comments");
				jtp_Script.repaint();
			}
		});	
		
		// Separator
		JLabel s1 = new JLabel();
		s1.setBorder(new CompoundBorder(
			new EmptyBorder(0,4,0,4),
			new MatteBorder(15,2,15,1,Constants.COLOR_BACKDROP)));
		
		menuPane.add(Box.createHorizontalGlue());
		menuPane.add(insertLine);
		menuPane.add(s1);
		menuPane.add(m_showComments);
		menuPane.add(m_commentMode);
		menuPane.add(m_clearComments);
		menuPane.add(Box.createHorizontalGlue());
		menuPane.setBorder(new CompoundBorder(
				new MatteBorder(10,ScriptPane.ICON_SPIRAL_WIDTH,0,10,
						Constants.COLOR_BACKDROP),
				new LineBorder(Color.black)
		));
		
		// Create and add a blank ScriptPane
		jtp_Script = new ScriptPane(this);
		
		rightSide = new JPanel(new BorderLayout());
		rightSide.add(menuPane, BorderLayout.NORTH);
		rightSide.add(jtp_Script, BorderLayout.CENTER);
		
		jtp_Script.setupScriptPane(getSize().height - OFFSET);
		
		setRightComponent(rightSide);
	}
	
	/**
	 * Adds a character to the story.  This function has been replaced
	 * with m_parent.addCharacter(), which calls addCharacter(gc).
	 * 
	 * @deprecated
	 *
	 */
	public void addCharacter(){
		// Steps in adding a character:
		// 1) Create a new GlobalCharacter (other some new char?)
		// 2) Add character to Story
		// 3) Add blank label to m_gallery
		
		// 1) Create a new globalcharacter
		GlobalCharacter gc = new GlobalCharacter(getStory().getCharSize()); 
		gc.setName("New Character");
		
		// 2)
		gc.setCharID(getStory().addCharacter(gc), getStory().getCharSize());
		
		// 3)
		gc.makeThumbnail(Image.SCALE_FAST);
		addCharacter(gc);
	}
	
	/**
	 * Adds a new character, gc, to the gallery.
	 * 
	 * @param gc = GlobalCharacter to be added.
	 */
	public void addCharacter(GlobalCharacter gc){
		// Add to gallery
		ScriptListButton button = new ScriptListButton(
				gc.getName(), gc.getID(), m_gallery, true);
		m_gallery.addItemToList(button);

		m_gallery.constructList(getStory().getCharacterOrder());
		m_gallery.selectButton();
		m_gallery.revalidate();
		
		// Add to scriptPane's combo boxes.
		jtp_Script.addCharacter(gc.getID(), gc.getName());
	}
	
	/**
	 * Renames a character in the gallery and ScriptPane.
	 * 
	 * @param id Character's ID
	 * @param name Character's new name
	 */
	public void renameCharacter(int id, String name){
		GlobalCharacter gc = getStory().getCharacter(id);
		gc.setName(name);
		
		((CharacterUI)m_parent.getInterface(Constants.CHARACTER_INDEX)).
			renameCharacter(id, name);
		
		jtp_Script.renameCharacter(id, name);
	}
	
	/**
	 * Deletes character from Script UI's gallery and the ScriptPane.
	 * NO CONFIRMATION
	 * 
	 * @param gid = Character's global id
	 */
	public void deleteCharacter(int gid){
		// Delete from ScriptUI
		m_gallery.removeItemFromList(gid);
		
		jtp_Script.deleteCharacter(gid);
	}
	
	/**
	 * Guesses who the next speaker will be, based on the previously
	 * defined script elements.  
	 * 
	 * In this simple version, we try to alternate two speakers, regardless
	 * of the number of speakers in the scene/story.
	 * 
	 * @return ID of 'next' character
	 */
	public int guessNextSpeaker(){
		// Case 1: No previous dialog.  Return 0
		if(jtp_Script.getDialogCount() == 0)
			return 0;
		else if(jtp_Script.getDialogCount() > 1)
			return jtp_Script.getAlternatingSpeaker();
		else if (jtp_Script.getLastSpeaker() > 0)
			return 0;
		else if (jtp_Script.getCharacterCount() > 1)
			return 1;
		else
			return 0;
	}
	
	/**
	 * Updates the gallery by calling constructList.
	 *
	 */
	public void updateGallery(){
		m_gallery.constructList(getStory().getCharacterOrder());
	}
	
	/**
	 * Clears the ScriptPane, but does nothing to the gallery for some
	 * reason.
	 *
	 */
	public void newStory(){
		jtp_Script.clearScript();
	}
	
	/**
	 * Opening a story is identical to opening the Script tab.
	 *
	 */
	public void openStory(){
		openTab();
	}
	
	public void openTab(){
		// Steps in opening the Script tab:
		// 1) Synch up characters.  All and only characters
		//    present in the char tab should appear here.
		//    Any characters that have been deleted should
		//    be doubly deleted.
		// FACT: getStory().getCharacters() should work for
		//       both ScriptUI and CharacterUI.
		// 2) Display updated script
		m_gallery.clearList();
		jtp_Script.clearScript();
		
		// 1)
		ArrayList charOrder = getStory().getCharacterOrder();
		for(int i=0; i<charOrder.size(); i++){
			Integer id = (Integer)charOrder.get(i);
			GlobalCharacter gc = getStory().getCharacter(id.intValue());
			ScriptListButton slb = new ScriptListButton(
					gc.getName(), gc.getID(), m_gallery, true);
			m_gallery.addItemToList(slb);
			jtp_Script.addCharacter(gc.getID(), gc.getName());
		}
		
		m_gallery.constructList(getStory().getCharacterOrder());
		
		// 2)
		jtp_Script.populateScriptPane(getStory().getDialogs());
		jtp_Script.repaint();
	}
	
	/**
	 * Handles tab closing by updating the stored Script elements
	 * in the Story.
	 *
	 */
	public void closeTab(){
		// Store dialog to story
		jtp_Script.updateStory();
	}
	
}
