/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Windows;

import javax.swing.*;
import java.awt.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Actions.*;
import edu.uiuc.orchid.clover.Interface.ActionGrid.ActionButton;

/**
 * EmoteDialog:
 *  Implements the dialog/toolbox for expressing emotions.
 * 
 * @author Terry J Bradley
 * @version 04.8.17
 *
 */
public class EmotionDialog extends JDialog implements Constants {

	protected CloverWindow m_parent;	// CloverWindow parent
	protected int m_emotion = -1;		// Current emotion

	/**
	 * Constructor - Creates an emotion toolbox with a
	 * single row of emotion action buttons.
	 * 
	 * @param owner CloverWindow parent
	 */
	public EmotionDialog(CloverWindow owner) {	
		super(owner, false);
		setTitle("Emotion Toolbox");
		m_parent = owner;
		
		setDefaultCloseOperation(JDialog.HIDE_ON_CLOSE);
		setResizable(false);
		
		JPanel emotion = new JPanel();
		emotion.setLayout(new GridLayout(1,8));

		for( int i=0; i<Emote.EMOTICONS.length; i++ ) {
			ActionButton jtb = new ActionButton(0, null, 
					Constants.EMOTE_OFFSET + 
					Constants.EMOTION_DIVIDER * Emote.EMOTICONS[i].m_index, 
					Emote.EMOTICONS[i].m_image,
					"", true, m_parent);
			jtb.setEmotion(Emote.EMOTICONS[i].m_index);
			jtb.setName(""+Emote.EMOTICONS[i].m_index);
			jtb.setToolTipText(Emote.EMOTICONS[i].m_description);
			
			emotion.add(jtb);
		}

		getContentPane().add(emotion);

		pack();
	}
	
	/**
	 * Unique show function allows custom placement of the toolbox.
	 * 
	 * @param location Location of toolbox.
	 */
	public void show(Point location) {
		pack();
		setLocation(location);
		
		// Toggle visibility
		if( isVisible() )
			super.hide();
		else
			super.show();
	}
		
}