/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Windows;

import javax.swing.*;
import java.awt.event.*;
import java.util.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.CloverList.*;
import edu.uiuc.orchid.clover.Interface.Tabs.CharacterUI;
import edu.uiuc.orchid.clover.Interface.DragGlassPane;
;

/**
 * GalleryDialog class - Implements the dialog version of the gallery
 * of characters.  Dragging and dropping a character onto a scene will
 * add that character to the scene.
 * 
 * @author Terry Bradley
 * @version 04.8.18 
 * @date 9/23/03
 *
 */
public class GalleryDialog extends JDialog  {

	protected CloverWindow m_parent;	// CloverWindow parent
	protected CloverList m_gallery;		// Gallery of character thumbnails
	
	int NUM_COLS = 5;					// Number of columns to use (dynamic)
	
	/**
	 * Constructor - Create an empty character gallery.
	 * 
	 * @param par CloverWindow parent
	 */
	public GalleryDialog(CloverWindow par) { 
		super(par, "Character Gallery", false);
		m_parent = par;
		
		setDefaultCloseOperation(JDialog.DO_NOTHING_ON_CLOSE);
		setGlassPane(new DragGlassPane());
		
		m_gallery = new CloverList(m_parent, false, null);
		m_gallery.setOrdered(false);
		getContentPane().add(m_gallery);

		setSize(Constants.GALLERY_DIALOG_WIDTH, Constants.GALLERY_DIALOG_HEIGHT);
		
		// Reconstruct the gallery on resize to dynamically change the layout
		addComponentListener(new ComponentAdapter(){
			public void componentResized(ComponentEvent e){
				try {
					m_gallery.constructList(m_parent.getStory().getCharacterOrder());
				}
				catch (NullPointerException ex){}
			}
		});
	}
	
	/**
	 * Sets the gallery data and rebuilds the gallery.
	 * 
	 * @param v Hashtable of data
	 */
	public void setGallery(Hashtable v){
		m_gallery.setListData(v);
		m_gallery.constructList(m_parent.getStory().getCharacterOrder());
	}
	
	/**
	 * Overwritten setVisible function creates its own Hashtable
	 * from Story data and sets the gallery.
	 */
	public void setVisible(boolean vis){
		if( vis ){
			Hashtable v = new Hashtable();
			Hashtable data = ((CharacterUI)m_parent.getInterface(
					Constants.CHARACTER_INDEX)).m_gallery.getListData();
			Enumeration e=data.elements();
			while( e.hasMoreElements() ){
				CharacterButton cb = (CharacterButton)e.nextElement();
				CharacterDialogButton cdb = new CharacterDialogButton(
					(ImageIcon)(cb.getIcon()), 
					CloverWindow.stripHTML2(cb.getText()),
					cb.getID(), 
					m_gallery, true, m_parent, this);
				v.put(new Integer(cdb.getID()), cdb);
			}
			setGallery(v);
		}
		super.setVisible(vis);
	}
	
}
