/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

import java.util.ArrayList;

import edu.uiuc.orchid.clover.Data.*;

/**
 * ProcessScene class - Processing thread for the Scene level.  Go-between
 * for switching scenes AND actions.
 * 
 * @author Terry J Bradley
 * @version 04.8.19
 *
 */
public class ProcessScene extends Thread {
	private ActionSynch actSynch;	// Action Synch producer/consumer
	private SceneSynch sceneSynch;	// Scene Synch producer/consumer
	// Both of the above synch instances are shared with the other process
	// threads (see ProcessAction and ProcessStory.java)
	
	private Scene m_scene;			// Current scene
	private Playable play;			// Playable interface
	private boolean stop = false;	// Run until stop is true

	/**
	 * Constructor - Sets action Synch, scene Synch, and interface.
	 * 
	 * @param c Action Synch variable
	 * @param s Scene Synch variable
	 * @param p Playable interface
	 */
	public ProcessScene(ActionSynch c, SceneSynch s, Playable p) {
		actSynch = c;
		sceneSynch = s;
		play = p;
	}

	/**
	 * Outer while loop processes scenes until none are left, at which point
	 * the thread halts.  For each scene, a double for loop puts actions
	 * in the synch instance column by column and row by row. 
	 */
	public void run() {
		while(!stop){
			m_scene = sceneSynch.getScene();
			if(m_scene == null){
				stop = true;
				actSynch.putCell(0, 0, 0, true);
				return;
			}
			play.setBackgroundImage(m_scene.getSceneID());
			ArrayList actions = m_scene.getActions();
			ArrayList characters = (ArrayList)actions.get(0);
			play.hideCharacters();
			for(int i=0; i<characters.size(); i++){
				int gid = ((Long)characters.get(i)).intValue();
				try {
					LocalCharacter lc = m_scene.getCharacter(gid);
					play.setCharacter(lc.getLocation(),
						lc.getSize(), gid, true);
				} catch (Exception e){/*ignore misses*/}
			}
			
			// traverse by columns
			for(int i=1; i<actions.size(); i++){
				ArrayList colActions = (ArrayList)actions.get(i);
				// traverse by row
				for(int j=0; j<colActions.size(); j++){
					// row = j, col = i
					Integer act = (Integer)colActions.get(j);
					if(act.intValue() > 0) {
						actSynch.putCell(m_scene.getSceneID(), j, i, false);
					}
					else
						// Hide emotion.  Notice that this makes above if clause pointless ;)
						actSynch.putCell(m_scene.getSceneID(), j, i, false);
				}//for
			}//for
		}//while
	}//run
	
}