/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Actions;

import java.io.*;
import edu.uiuc.orchid.clover.*;

/**
 * Emote.  Currently the only available SecondaryAction, an Emote action 
 * is defined by a name and appearance.  May later expand into visual/audio
 * depictions.
 * 
 * @author Terry J Bradley
 * @version 04.7.26
 *
 */

public class Emote extends SecondaryAction implements Constants {
	
	public int m_emotion;		// See Constants for enumeration of emotions.
	
	/**
	 * Static array contains all emotions.  Commented emotions are
	 * expected to be added in future versions.
	 */
	public static Emoticon[] EMOTICONS = {
		new Emoticon(Clover.EMOTION_NONE_LOC, "None", EMOTION_NONE),
		new Emoticon(Clover.EMOTION_HAPPY_LOC, "Happy", EMOTION_HAPPY),
		new Emoticon(Clover.EMOTION_SAD_LOC, "Sad", EMOTION_SAD),
		//new Emoticon(Clover.EMOTION_EXCITED_LOC, "Excited", EMOTION_EXCITED),
		new Emoticon(Clover.EMOTION_ANGRY_LOC, "Angry", EMOTION_ANGRY),
		//new Emoticon(Clover.EMOTION_INNOCENT_LOC, "Innocent", EMOTION_INNOCENT),
		new Emoticon(Clover.EMOTION_CONFUSED_LOC, "Confused", EMOTION_CONFUSED),
		new Emoticon(Clover.EMOTION_LOVE_LOC, "Love", EMOTION_LOVE),
		new Emoticon(Clover.EMOTION_SURPRISED_LOC, "Surprised", EMOTION_SURPRISED),
		//new Emoticon(Clover.EMOTION_FLOORED_LOC, "Floored", EMOTION_FLOORED),
		//new Emoticon(Clover.EMOTION_STARTLED_LOC, "Startled", EMOTION_STARTLED),
		new Emoticon(Clover.EMOTION_EMBARRASSED_LOC, "Embarrassed", EMOTION_EMBARRASSED),
		//new Emoticon(Clover.EMOTION_SARCASTIC_LOC, "Sarcastic", EMOTION_SARCASTIC),
		//new Emoticon(Clover.EMOTION_SLEEPY_LOC, "Sleepy", EMOTION_SLEEPY),
		//new Emoticon(Clover.EMOTION_STRESSED_LOC, "Stressed", EMOTION_STRESSED),
		//new Emoticon(Clover.EMOTION_HOT_LOC, "Hot", EMOTION_HOT),
		//new Emoticon(Clover.EMOTION_COLD_LOC, "Cold", EMOTION_COLD),
		//new Emoticon(Clover.EMOTION_NAUGHTY_LOC, "Naughty", EMOTION_NAUGHTY),
		//new Emoticon(Clover.EMOTION_NERVOUS_LOC, "Nervous", EMOTION_NERVOUS),
		//new Emoticon(Clover.EMOTION_LAUGH_LOC, "Laugh", EMOTION_LAUGH),
		new Emoticon(Clover.EMOTION_WINK_LOC, "Wink", EMOTION_WINK)
	};
	
	/**
	 * Constructor
	 * 
	 * @param emotion Emotion index
	 */
	public Emote(int emotion){
		m_emotion = emotion;
	}
	
	/**
	 * Constructor assigns Happy emotion
	 *
	 */
	public Emote(){
		m_emotion = EMOTICONS[0].m_index;
	}

	
// METHODS
	
	/**
	 * Sets emotion index to emo
	 * 
	 * @param emo Emotion index
	 */
	public void setEmotion(int emo) {
		m_emotion = emo;
	}
	
	/**
	 * Returns emotion index
	 * 
	 * @return Emotion index
	 */
	public int getEmotion() {
		return m_emotion;
	}
	
	/**
	 * Custom toString() function used to print name of emotion.
	 */
	public String toString() {
		return " while becoming "+Emote.getEmoticon(m_emotion).m_description;
	}
	
	/**
	 * Returns emotion matching index
	 * 
	 * @param emotion Index to match
	 * @return Emoticon corresponding to index
	 */
	public static Emoticon getEmoticon(int emotion){
		for( int i=0; i<EMOTICONS.length; i++ ){
			if( EMOTICONS[i].m_index == emotion ) {
				return EMOTICONS[i];
			}
		}
		return null;
	}
	
	/**
	 * Serialization functions.  Calling defaultWriteObject is
	 * probably sufficient.
	 * 
	 * @param oos Output stream
	 * @throws IOException Propogate errors to calling function.
	 */
	private void writeObject(ObjectOutputStream oos) throws IOException {
		oos.defaultWriteObject();
		oos.writeInt(m_emotion);
	}
	
	/**
	 * Serialization functions.  Calling defaultReadObject is
	 * probably sufficient.
	 * 
	 * @param ois Input stream
	 * @throws IOException Propogate errors to calling function.
	 */
	private void readObject(ObjectInputStream ois) throws IOException  {
		try {
			ois.defaultReadObject();
			m_emotion = ois.readInt();
		} catch (ClassNotFoundException ce){}		
	}

}