/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Actions;

import java.util.ArrayList;
import java.io.*;


/**
 * Speak.  One of two primary actions.  The PrimaryAction class
 * handles most functions - this class is primarily used to distinguish
 * between Speak and Think actions.
 * 
 * @author Terry J Bradley
 * @version 04.7.26
 *
 */

public class Speak extends PrimaryAction {
	
	String m_audioPath = null;	// Used to identify audio file.
	
// Constructors	
	public Speak(int source, int target, int time) {
		super(source, target, time);
	}
	
	public Speak(int source, int target, int time, String dialog) {
		super(source, target, time);
		m_dialog = dialog;
	}
	
	public Speak(int time) {
		super(time);
	}
	
// Methods	
	/**
	 * Sets the audio path
	 */
	public void setAudioPath(String p){
		m_audioPath = p;
	}
	
	/**
	 * Returns the audio path
	 */
	public String getAudioPath(){
		return m_audioPath;
	}
	
	/**
	 * This method might guess the location of a character's head to 
	 * better place a call-out bubble.  Not currently implemented or 
	 * used.
	 */
	public void guessLocation() {}
	
	/**
	 * Smarter toString method prints character name and action details.
	 */
	public String toString() {
		return super.toString() + "says " + m_dialog;
	}
	
	/**
	 * Serialization methods are needed because of the added variable
	 * for speech audio.
	 * 
	 * @param oos
	 * @throws IOException
	 */
	private void writeObject(ObjectOutputStream oos) throws IOException {
		oos.defaultWriteObject();
		//super.writeObject(oos);
		oos.writeObject(m_targetGID);
		oos.writeObject(m_secondary);
		oos.writeObject(m_dialog);
		oos.writeInt(m_sourceGID);
		oos.writeInt(m_startingCol);
		oos.writeInt(m_minDuration);
		oos.writeInt(m_length);
		
		oos.writeObject(m_audioPath);
	}
	
	private void readObject(ObjectInputStream ois) throws IOException  {
		try {
			ois.defaultReadObject();
			
			m_targetGID		= (ArrayList)ois.readObject();
			m_secondary		= (SecondaryAction)ois.readObject();
			m_dialog		= (String)ois.readObject();
			m_sourceGID		= ois.readInt();
			m_startingCol	= ois.readInt();
			m_minDuration	= ois.readInt();
			m_length		= ois.readInt();
			
			m_audioPath		= (String)ois.readObject();
		} catch (ClassNotFoundException ce){
			System.err.println(ce);
		}		
	}

	
	
}