/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/
 
package edu.uiuc.orchid.clover;

import java.awt.*;
import java.io.*;
//import java.net.URLConnection;
import java.nio.channels.FileChannel;

import javax.swing.ImageIcon;
import javax.swing.*;
import javax.imageio.*;

//import edu.uiuc.orchid.clover.Interface.PreviewBox;
import edu.uiuc.orchid.clover.Interface.Windows.*;

/**
 * Clover Application.  FOR LICENSE AND COPYRIGHT INFORMATION, PLEASE
 * SEE LICENSE.TXT IN CLOVER'S ROOT DIRECTORY.
 * 
 * Create and display a single CloverWindow.
 * Also includes a handful of useful static functions,
 * directories, and file locations.
 * 
 * @author Terry Bradley
 * @version 04.7.26
 * 
 * */
///////////////////////////////////////////////////////////////////////////////

public class Clover implements Constants {
	
	/**
	 * The .userpref file is used to store settings between instances
	 * of the Clover application.
	 */
	private static final String PREF_FILENAME = ".userpref";
	
	public static boolean PLATFORM;		// True = Windows, False = Other
	public static boolean TABLET;		// True = Wacom attached
	
	/**
	 * This value determines whether Clover will use the Windows
	 * look and feel (if available) or the Java / Metal plaf.
	 */
	public static final boolean WIN_LOOK_AND_FEEL = true;
	
	/**
	 * Directory overview:
	 * 
	 * When Clover is run, all the resources, temp files, and story locations
	 * are computed relative to the working directory.  USER_DIR specifies 
	 * this working directory, and is assigned during the Main() function.
	 * All the other directories and file locations are derived from this
	 * directory.
	 * 
	 * Important directories:
	 * 
	 * USER_DIR		- Working directory
	 * TEMP_DIR		- USER_DIR/Temp, stores files for open story
	 * PREVIEW_DIR	- USER_DIR/Temp/Preview, stores 'preview.png' for
	 * 				  story preview functionality.
	 * STORY_DIR	- USER_DIR/Stories, Default location for new/existing
	 * 				  stories.
	 * IMAGE_DIR	- USER_DIR/Resources/images, Root location for all
	 * 				  images, including icons, cursors, speech bubbles, etc.
	 */
	public static String USER_DIR;
	public static String TEMP_DIR;	
	public static String PREVIEW_DIR;		
	public static String STORY_DIR;
	public static String IMAGE_DIR;
	public static String ICON_DIR;
	public static String CURSOR_DIR;
	public static String EMOTICON_DIR;
	public static String BUBBLES_DIR;
	public static String SCRIPTPANE_DIR;
	public static String BRUSH_ICON_LOC;
	public static String FILL_ICON_LOC;
	public static String ERASER_ICON_LOC;
	public static String ERASE_CURSOR_LOC;
	public static String FILL_CURSOR_LOC;
	public static String BRUSH_CURSOR_LOC;
	public static String CLOVER_SPLASH_ICON_LOC;
	public static String CLOVER_TITLE_ICON_LOC;
	public static String SPEAK_ICON_LOC;
	public static String THINK_ICON_LOC;
	public static String EMOTE_ICON_LOC;
	public static String TRASH_ICON_LOC;
	
	public static String EMOTION_NONE_LOC;
	public static String EMOTION_HAPPY_LOC; 
	public static String EMOTION_SAD_LOC;
	public static String EMOTION_EXCITED_LOC;
	public static String EMOTION_ANGRY_LOC;
	public static String EMOTION_INNOCENT_LOC;
	public static String EMOTION_CONFUSED_LOC;
	public static String EMOTION_LOVE_LOC;
	public static String EMOTION_SURPRISED_LOC;
	public static String EMOTION_FLOORED_LOC;
	public static String EMOTION_STARTLED_LOC;
	public static String EMOTION_EMBARRASSED_LOC;
	public static String EMOTION_SARCASTIC_LOC;
	public static String EMOTION_SLEEPY_LOC;
	public static String EMOTION_STRESSED_LOC;
	public static String EMOTION_AFRAID_LOC;
	public static String EMOTION_HOT_LOC;
	public static String EMOTION_COLD_LOC;
	public static String EMOTION_NAUGHTY_LOC;
	public static String EMOTION_NERVOUS_LOC;
	public static String EMOTION_LAUGH_LOC;
	public static String EMOTION_WINK_LOC;
	
	public static String EMOTION_HAPPY_ACTIVE_LOC;
	public static String EMOTION_SAD_ACTIVE_LOC;
	public static String EMOTION_ANGRY_ACTIVE_LOC;
	public static String EMOTION_CONFUSED_ACTIVE_LOC;
	public static String EMOTION_LOVE_ACTIVE_LOC;
	public static String EMOTION_SURPRISED_ACTIVE_LOC;
	public static String EMOTION_EMBARRASSED_ACTIVE_LOC;
	public static String EMOTION_WINK_ACTIVE_LOC;
	
	public static String EMOTION_HAPPY_STATIC_LOC;
	public static String EMOTION_SAD_STATIC_LOC;
	public static String EMOTION_ANGRY_STATIC_LOC;
	public static String EMOTION_CONFUSED_STATIC_LOC;
	public static String EMOTION_LOVE_STATIC_LOC;
	public static String EMOTION_SURPRISED_STATIC_LOC;
	public static String EMOTION_EMBARRASSED_STATIC_LOC;
	public static String EMOTION_WINK_STATIC_LOC;
	
	public static String EMOTION_HAPPY_AUDIO_LOC;
	public static String EMOTION_SAD_AUDIO_LOC;
	public static String EMOTION_ANGRY_AUDIO_LOC;
	public static String EMOTION_CONFUSED_AUDIO_LOC;
	public static String EMOTION_LOVE_AUDIO_LOC;
	public static String EMOTION_SURPRISED_AUDIO_LOC;
	public static String EMOTION_EMBARRASSED_AUDIO_LOC;
	public static String EMOTION_WINK_AUDIO_LOC;
	
   	public static String SPEECH_BUBBLE_0_LOC;
   	public static String SPEECH_BUBBLE_1_LOC;
   	public static String SPEECH_BUBBLE_2_LOC;
   	public static String SPEECH_BUBBLE_3_LOC;
   	
	public static String SPEECH_BUBBLE_LEFT_0_LOC;
	public static String SPEECH_BUBBLE_LEFT_1_LOC;
	public static String SPEECH_BUBBLE_LEFT_2_LOC;
	public static String SPEECH_BUBBLE_LEFT_3_LOC;
   
   	public static String THINK_BUBBLE_0_LOC;
  	public static String THINK_BUBBLE_1_LOC;
   	public static String THINK_BUBBLE_2_LOC;
   	public static String THINK_BUBBLE_3_LOC;
   	
	public static String THINK_BUBBLE_LEFT_0_LOC;
	public static String THINK_BUBBLE_LEFT_1_LOC;
	public static String THINK_BUBBLE_LEFT_2_LOC;
	public static String THINK_BUBBLE_LEFT_3_LOC;
	
	/**
	 * Returns CloverProperties data from the file specified
	 * in PREF_FILENAME.
	 * 
	 * @return CloverProperties data from the file specified in PREF_FILENAME
	 */
	public static CloverProperties getUserPrefs() {
		CloverProperties userPrefs = new CloverProperties();
		FileInputStream in;

		// now load properties from last invocation
		try {
			in = new FileInputStream(PREF_FILENAME);
			userPrefs.load(in);
			in.close();
		}
		catch (FileNotFoundException e) { userPrefs = setDefaultPrefs(); }
		catch (IOException f) { userPrefs = setDefaultPrefs(); }
		return userPrefs;	
	}
	
	
	/**
	 *  Called if getUserPrefs() fails.  Creates a new
	 *  set of Properties and assigns a few values.
	 * 
	 * 	If the user preferences file cannot be loaded, set default
	 *  values for screen dimensions.  Other defaults will be added
	 *  to prefs as needed.  On exit, all properties will be written
	 *  to the preferences file.
	 */
	public static CloverProperties setDefaultPrefs() {
		CloverProperties userPrefs = new CloverProperties();
		Dimension dim = Toolkit.getDefaultToolkit().getScreenSize();
		
		userPrefs.setProperty("App.fScreenWidth", 
			Integer.toString(dim.width));
		userPrefs.setProperty("App.fScreenHeight", 
			Integer.toString(dim.height));
			
		System.err.println("Loading from .userprefs file failed - " +
				"using defaults.");
			
		return userPrefs;
	}
	
	/**
	 * Writes "prefs" to the file at PREF_FILENAME
	 * 
	 * @param prefs CloverProperties to store to disk
	 */
	public static void writeUserPrefs(CloverProperties prefs) {
		try {
			FileOutputStream out = new FileOutputStream(PREF_FILENAME);
			prefs.store(out, "Header");
			out.close();
		}
		catch (FileNotFoundException e) {}
		catch (IOException f) {}
	}
	

	
	/**
	 * Returns an ImageIcon, or null if the path was invalid.
	 * Prior versions of this method used java.net.URL instead
	 * of String.  This was found to cause problems when running
	 * from an executable jar, hence the switch.
	 *  
	 * @param path Path to .gif or other image file
	 * @return ImageIcon based on image at path
	 */
	public static ImageIcon createImageIcon2(String path) {
		ImageIcon ii = null;
		try {
			ii = new ImageIcon(path);
		} catch (Exception e){
			e.printStackTrace();
			ii = null;
		}
		if (ii == null || ii.getIconHeight() < 0) {
			//System.err.println("Error loading image - "+path);
		}
		return ii;
	}
	
	/**
	 * Returns an ImageIcon, ignoring the cache.  Here, width and height
	 * must represent the dimensions of the Image - resizing is not supported.
	 * 
	 * @param path Path to image
	 * @param width Image width
	 * @param height Image height
	 * @return Image or null if path is invalid.
	 */
	public static ImageIcon createPreviewImageIcon(String path, int width,
			int height)
	{
		int MAX_IMAGE_SIZE = width * height * 4;
		java.net.URL url = null;
		java.net.URLConnection con;
		int count = 0;
		try {
			url = new java.net.URL("file:///"+path);
		    con = url.openConnection();
		    con.setUseCaches(false);
		    BufferedInputStream bis = new BufferedInputStream(con.getInputStream());
		    int data;
		    byte[] imgData = new byte[MAX_IMAGE_SIZE];
		    count = bis.read(imgData);
		    bis.close();
		    if(count <= 0)
		    	return null;
		    return new ImageIcon(Toolkit.getDefaultToolkit().createImage(imgData));
		} catch (Exception e1) {
		    //e1.printStackTrace();
		    return null;
		}
	}
	
	/**
	 * Returns an ImageIcon, resized to width x height, or null
	 * if the path was invalid.
	 *  
	 * @param path Path to .gif or other image file
	 * @param width Image width
	 * @param height Image height
	 * @return ImageIcon based on image at path
	 */
	public static ImageIcon createImageIcon(String path, int width, 
											   int height)
	{
		String completePath = //System.getProperty("user.dir") + 
			path;
		try{
			return new ImageIcon( ImageIO.read(new File(completePath)).
				getScaledInstance(width, height, Image.SCALE_AREA_AVERAGING) );
		}
		catch(Exception e){
			//System.err.println("Couldn't find file: " + path);
			return null;
		}
	}
	
	/**
	 * Copies a file from source to destination.
	 * 
	 * @param source
	 * @param destination
	 */
	public static void copyFile(String source, String destination){
		
		FileChannel in = null;
		FileOutputStream fos = null;
		File file = new File(destination);
		
		// Create destination file
		try {
			file.createNewFile();
			fos = new FileOutputStream(file);
			in = fos.getChannel();
		} catch(Exception e){
			System.err.println("Error creating "+destination);
		}
		
		// Read and copy file contents.
		try {
			FileInputStream fis = new FileInputStream(source);
			FileChannel fc = fis.getChannel();
			fc.transferTo(0l, fc.size(), in);
			
			// Close everything
			in.close();
			fos.close();
			fc.close();
			fis.close();
		} catch(Exception e){
			System.err.println("Error copying "+source);
		}
		
	}
	
	/**
	 * Returns path minus filename.extension
	 * 
	 * @param path
	 * @return
	 */
	public static String getDirectory(String path){
		return path.substring(0, path.lastIndexOf('\\')) + "\\";
	}
	
	
	public static void main(String[] args) {
		// Set look and feel
		
		if( WIN_LOOK_AND_FEEL ) {
			try {
				UIManager.setLookAndFeel(
					UIManager.getSystemLookAndFeelClassName());
			}
			catch (Exception e) {
				System.err.println("Couldn't load Windows look and feel.");
			}
		}
		
		// Check for Windows
		if( System.getProperty("os.name").startsWith("Windows") ){
			Clover.PLATFORM = Constants.PLATFORM_WINDOWS;
			
		// Check for Wacom tablet
			/*	File file = new File("jwintab.dll");
				Clover.TABLET = file.exists(); 			*/
			//Jwintab.open();
		}
		else
			Clover.PLATFORM = Constants.PLATFORM_NOT_WINDOWS;
		
			
		// Turn on Java Windows
		JFrame.setDefaultLookAndFeelDecorated(true);
		JDialog.setDefaultLookAndFeelDecorated(true);
				
		// Load preferences file
		CloverProperties userPrefs = getUserPrefs();
		
		// Setup directories for rest of Clover to use.
		// See the declarations above for more information.
		File f = new File(".");
		String userdir = f.getAbsolutePath();

		// TODO: However, a better approach would be to include your 
		// images directory in the application jar file and use the 
		// classloader to get the file, i.e:
		//
		// 	URL url = ClassLoader.getSystemResource("images/image.gif");
		//  Image image = Toolkit.getDefaultToolkit().getImage(url);
		
		//USER_DIR 				= System.getProperty("user.dir")+"\\";
		USER_DIR				= userdir.substring(0, userdir.length() - 1);
		TEMP_DIR 				= USER_DIR + "Temp\\";
		PREVIEW_DIR 			= TEMP_DIR + "Preview\\";
		STORY_DIR 				= USER_DIR + "Stories\\";
		IMAGE_DIR 				= USER_DIR + "Resources\\images\\";
		BUBBLES_DIR 			= IMAGE_DIR + "Bubbles\\";
		CURSOR_DIR 				= IMAGE_DIR + "Cursors\\";
		EMOTICON_DIR 			= IMAGE_DIR + "Emoticons\\";
		ICON_DIR 				= IMAGE_DIR + "Icons\\";
		SCRIPTPANE_DIR 			= IMAGE_DIR + "ScriptPane\\";		
		
		CLOVER_SPLASH_ICON_LOC 	= ICON_DIR+"clover-sketchicon_smaller.gif";
		CLOVER_TITLE_ICON_LOC 	= ICON_DIR+"clover-sketchicon16x16.gif";
		
		BRUSH_ICON_LOC 			= ICON_DIR+"pencil_cursor32.gif";
		FILL_ICON_LOC 			= ICON_DIR+"bucket_morecursor32.gif";
		ERASER_ICON_LOC			= ICON_DIR+"eraser_icon.png";
		ERASE_CURSOR_LOC 		= CURSOR_DIR+"erase_bwcursor32.gif";
		FILL_CURSOR_LOC 		= CURSOR_DIR+"bucket_crosscursor32.gif";
		BRUSH_CURSOR_LOC 		= CURSOR_DIR+"pencil_cursor32.gif";
		
		SPEAK_ICON_LOC			= ICON_DIR+"speak_icon2.gif";
		THINK_ICON_LOC			= ICON_DIR+"think_icon2.gif";
		EMOTE_ICON_LOC			= ICON_DIR+"emote_icon2.gif";
		TRASH_ICON_LOC			= ICON_DIR+"delete48.gif";
		
	   	SPEECH_BUBBLE_0_LOC			= BUBBLES_DIR+"speech_bubble_0.gif";
	   	SPEECH_BUBBLE_1_LOC			= BUBBLES_DIR+"speech_bubble_1.gif";
	   	SPEECH_BUBBLE_2_LOC			= BUBBLES_DIR+"speech_bubble_2.gif";
	   	SPEECH_BUBBLE_3_LOC			= BUBBLES_DIR+"speech_bubble_rounded.gif";
	   	
		SPEECH_BUBBLE_LEFT_0_LOC	= BUBBLES_DIR+"speech_bubble_left_0.gif";
		SPEECH_BUBBLE_LEFT_1_LOC	= BUBBLES_DIR+"speech_bubble_left_1.gif";
		SPEECH_BUBBLE_LEFT_2_LOC	= BUBBLES_DIR+"speech_bubble_left_2.gif";
		SPEECH_BUBBLE_LEFT_3_LOC	= BUBBLES_DIR+"speech_bubble_left_rounded.gif";
	   
	   	THINK_BUBBLE_0_LOC			= BUBBLES_DIR+"thought_bubble_0.gif";
	  	THINK_BUBBLE_1_LOC			= BUBBLES_DIR+"thought_bubble_1.gif";
	   	THINK_BUBBLE_2_LOC			= BUBBLES_DIR+"thought_bubble_2.gif";
	   	THINK_BUBBLE_3_LOC			= BUBBLES_DIR+"thought_bubble_rounded.gif";
	   	
		THINK_BUBBLE_LEFT_0_LOC		= BUBBLES_DIR+"thought_bubble_left_0.gif";
		THINK_BUBBLE_LEFT_1_LOC		= BUBBLES_DIR+"thought_bubble_left_1.gif";
		THINK_BUBBLE_LEFT_2_LOC		= BUBBLES_DIR+"thought_bubble_left_2.gif";
		THINK_BUBBLE_LEFT_3_LOC		= BUBBLES_DIR+"thought_bubble_left_rounded.gif";
		
		EMOTION_NONE_LOC		= EMOTICON_DIR+"indifferent.gif";//
		EMOTION_HAPPY_LOC		= EMOTICON_DIR+"happy.gif";// 
		EMOTION_SAD_LOC			= EMOTICON_DIR+"sad.gif";//
		EMOTION_EXCITED_LOC		= EMOTICON_DIR+"excited.gif";
		EMOTION_ANGRY_LOC		= EMOTICON_DIR+"angry.gif";//
		EMOTION_INNOCENT_LOC	= EMOTICON_DIR+"innocent.gif";
		EMOTION_CONFUSED_LOC	= EMOTICON_DIR+"confused.gif";//
		EMOTION_LOVE_LOC		= EMOTICON_DIR+"love.gif";//
		EMOTION_SURPRISED_LOC	= EMOTICON_DIR+"surprised.gif";//
		EMOTION_FLOORED_LOC		= EMOTICON_DIR+"floored.gif";
		EMOTION_STARTLED_LOC	= EMOTICON_DIR+"startled.gif";
		EMOTION_EMBARRASSED_LOC	= EMOTICON_DIR+"embarrassed.gif";//
		EMOTION_SARCASTIC_LOC	= EMOTICON_DIR+"sarcastic.gif";
		EMOTION_SLEEPY_LOC		= EMOTICON_DIR+"sleepy.gif";
		EMOTION_STRESSED_LOC	= EMOTICON_DIR+"stressed.gif";
		EMOTION_AFRAID_LOC		= EMOTICON_DIR+"afraid.gif";
		EMOTION_HOT_LOC			= EMOTICON_DIR+"hot.gif";
		EMOTION_COLD_LOC		= EMOTICON_DIR+"cold.gif";
		EMOTION_NAUGHTY_LOC		= EMOTICON_DIR+"naughty.gif";
		EMOTION_NERVOUS_LOC		= EMOTICON_DIR+"nervous.gif";
		EMOTION_LAUGH_LOC		= EMOTICON_DIR+"laugh.gif";
		EMOTION_WINK_LOC		= EMOTICON_DIR+"wink.gif";
		
		EMOTION_HAPPY_ACTIVE_LOC		= BUBBLES_DIR+"emotion_happy_act.gif";
		EMOTION_SAD_ACTIVE_LOC			= BUBBLES_DIR+"emotion_sad_act.gif";
		EMOTION_ANGRY_ACTIVE_LOC		= BUBBLES_DIR+"emotion_angry_act.gif";
		EMOTION_CONFUSED_ACTIVE_LOC		= BUBBLES_DIR+"emotion_confused_act.gif";
		EMOTION_LOVE_ACTIVE_LOC			= BUBBLES_DIR+"emotion_love_act.gif";
		EMOTION_SURPRISED_ACTIVE_LOC	= BUBBLES_DIR+"emotion_surprised_act.gif";
		EMOTION_EMBARRASSED_ACTIVE_LOC	= BUBBLES_DIR+"emotion_embarrassed_act.gif";
		EMOTION_WINK_ACTIVE_LOC			= BUBBLES_DIR+"emotion_wink_act.gif";
		
		EMOTION_HAPPY_STATIC_LOC		= BUBBLES_DIR+"emotion_happy_stc.gif";
		EMOTION_SAD_STATIC_LOC			= BUBBLES_DIR+"emotion_sad_stc.gif";
		EMOTION_ANGRY_STATIC_LOC		= BUBBLES_DIR+"emotion_angry_stc.gif";
		EMOTION_CONFUSED_STATIC_LOC		= BUBBLES_DIR+"emotion_confused_stc.gif";
		EMOTION_LOVE_STATIC_LOC			= BUBBLES_DIR+"emotion_love_stc.gif";
		EMOTION_SURPRISED_STATIC_LOC	= BUBBLES_DIR+"emotion_surprised_stc.gif";
		EMOTION_EMBARRASSED_STATIC_LOC	= BUBBLES_DIR+"emotion_embarrassed_stc.gif";
		EMOTION_WINK_STATIC_LOC			= BUBBLES_DIR+"emotion_wink_stc.gif";
		
		EMOTION_HAPPY_AUDIO_LOC			= BUBBLES_DIR+"emotion_happy.wav";
		EMOTION_SAD_AUDIO_LOC			= BUBBLES_DIR+"emotion_sad.wav";
		EMOTION_ANGRY_AUDIO_LOC			= BUBBLES_DIR+"emotion_angry.wav";
		EMOTION_CONFUSED_AUDIO_LOC		= BUBBLES_DIR+"emotion_confused.wav";
		EMOTION_LOVE_AUDIO_LOC			= BUBBLES_DIR+"emotion_love.wav";
		EMOTION_SURPRISED_AUDIO_LOC		= BUBBLES_DIR+"emotion_surprised.wav";
		EMOTION_EMBARRASSED_AUDIO_LOC	= BUBBLES_DIR+"emotion_embarrassed.wav";
		EMOTION_WINK_AUDIO_LOC			= BUBBLES_DIR+"emotion_wink.wav";
		
		
		
		// Check if the Temp directory exists, if not, create it
		try {
			File cFile = new File(TEMP_DIR);
			if (!cFile.exists())
				cFile.mkdir();
		} catch (Exception e) {
		}
		
		
		// Create and display CloverWindow
		CloverWindow w;
		w = new CloverWindow(userPrefs);
		
		// Load story if one is passed in
		if(args != null && args.length > 0){
			String filename = args[0];
			
			// Do NOT need to add working dir to filename
			
			//w.openStory(new File(filename));
			// DEBUG
				//JOptionPane.showMessageDialog(w, "Attempting to pass in " +
				//		"the following argument: "+filename);
			// DEBUG
			w.openStory(new File(filename));
		}
		
		w.setVisible(true);
	}	
}