/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover;

import java.awt.*;

/**
 * The BIG file of Clover Constants.  First grouping gives keys
 * for the main CloverProperties file.  Followed by assorted constants.
 * 
 * @author Terry Bradley
 * @version 04.7.26
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public interface Constants
{
	
	/**
	 * Application settings for .userprefs.  Included are window
	 * sizes and locations.  SAVE_ON_EXIT allows save pop-up to be
	 * disabled (for debugging / testing).
	 */
	public static final String f_SCREEN_HEIGHT			= "App.fScreenHeight";
	public static final String f_SCREEN_WIDTH			= "App.fScreenWidth";
	public static final String f_TOOLBOX_WIDTH			= "App.fToolBoxWidth";
	public static final String f_TOOLBOX_HEIGHT			= "App.fToolBoxHeight";
	public static final String f_GALLERY_WIDTH			= "App.fGalleryWidth";
	public static final String f_GALLERY_HEIGHT			= "App.fGalleryHeight";
	public static final String d_SCREEN_X				= "App.dScreenX";
	public static final String d_SCREEN_Y				= "App.dScreenY";
	public static final String i_TOOLBOX_X				= "App.iToolBoxX";
	public static final String i_TOOLBOX_Y				= "App.iToolBoxY";
	public static final String i_GALLERY_X				= "App.iGalleryX";
	public static final String i_GALLERY_Y				= "App.iGalleryY";
	public static final String i_LEFT_PANEL_WIDTH		= "App.iPanelWidth";
	public static final String i_TB_WIDTH				= "App.iTBWidth";
	public static final String i_COMMENT_COLOR			= "App.iCommentColor";
	public static final String b_SAVE_ON_EXIT			= "App.bSaveChanges";
	
	// Location of last opened file.
	public static final String str_OPEN_PATH			= "App.strOpenPath";
	
	/**
	 * Hardcoded, optional setting allows characters/actions to
	 * be dragged on the initial click (rather than select first
	 * then drag).
	 * 
	 */
	public static final boolean ONE_CLICK_DRAG			= true;
	
	public static final String CLOVER_VERSION		 	= "04.10.11";
	
	// Platform info
	public static final boolean PLATFORM_WINDOWS		= true;
	public static final boolean PLATFORM_NOT_WINDOWS	= false;
	
	// Indices for TabbedPane:
	public final static int FILE_INDEX					= 0;
	public final static int NARRATIVE_INDEX				= 1;
	public final static int SCRIPT_INDEX				= 2;
	public final static int CHARACTER_INDEX				= 3;
	public final static int VIEW_AND_EDIT_INDEX			= 4;
	public final static int PLAYBACK_INDEX				= 5;
	public final static int RESPONSE_INDEX				= 6;
	
	// Click and Hold:
	public static final int POPUP_DELAY 				= 400; 	// msec
	public static final int TRAVEL 						= 20;	// px
	
	// Other Constants
	public static final boolean SELECTABLE 				= true;
	public static final boolean NOT_SELECTABLE 			= false;
	public static final boolean EMPTY 					= true;
	public static final boolean NOT_EMPTY 				= false;
	public static final boolean MALE 					= true;
	public static final boolean FEMALE 					= false;
		
//  COLORS
	public static final Color BG_COLOR					= new Color(154,154,154);
	public final static Color COLOR_BACKDROP			= Color.GRAY;
	public final static Color COLOR_LIST				= new Color(200,200,200);
	public final static Color COLOR_CANVAS 				= new Color(240,248,255);//Color.white;
	public static final Color COLOR_CLEAR				= new Color(0,0,0,0); //new Color(255,255,255,0);
	public static final Color COLOR_TABLE				= new Color(255,255,204);
	public static final Color COLOR_TABLE_SELECTION		= new Color(200,200,255);
	
// DIMENSIONS	
	public static final int CLABEL_DEFAULT_WIDTH		= 60;
	public static final int CLABEL_DEFAULT_HEIGHT		= 80;
	public final static int SCENE_THUMB_WIDTH 			= 96;
	public final static int SCENE_THUMB_HEIGHT 			= 60;
	public final static int SCENE_LIST_WIDTH			= 108;
	public final static int GALLERY_DIALOG_WIDTH		= 200;
	public final static int GALLERY_DIALOG_HEIGHT		= 400;
	
	/**
	 * Multiple sizes are given for scenes and character images.
	 * Larger displays will automatically get larger canvases
	 * for NEW stories ONLY.  Stories created on a large display
	 * will NOT play well with smaller displays and, to a lesser
	 * extent, vice versa.
	 * 
	 * TODO [3]: Correct this functionality by offering to resize
	 * images on load, when images are the "wrong size" 
	 */
	public final static int SCENECANVAS_1600_WIDTH		= 1265;
	public final static int SCENECANVAS_1600_HEIGHT		= 702;
	public final static int SCENECANVAS_1280_WIDTH		= 1012;
	public final static int SCENECANVAS_1280_HEIGHT		= 562; 
	public final static int SCENECANVAS_1024_WIDTH		= 810;
	public final static int SCENECANVAS_1024_HEIGHT		= 450;
	public final static int CHARCANVAS_1600_WIDTH		= 800;
	public final static int CHARCANVAS_1600_HEIGHT		= 1000;
	public final static int CHARCANVAS_1280_WIDTH		= 640;	
	public final static int CHARCANVAS_1280_HEIGHT		= 800; 
	public final static int CHARCANVAS_1024_WIDTH		= 496;
	public final static int CHARCANVAS_1024_HEIGHT		= 600;

//  DRAWING CONSTANTS	
	public static final int BRUSH_INDEX					= 0;
	public static final int FILL_INDEX					= 1;
	public static final int ERASE_INDEX					= 2;
	//public static final int SELECT_INDEX				= 3;

	public static final int BRUSH_SIZE					= 2;
	public static final int ERASOR_SIZE					= 10;
	
	/**
	 * Determines whether brush strokes throughout application are
	 * antialiased.  Note: turning this on leads to odd results
	 * with the fill tool.
	 */
	public static final Object BRUSH_ANTI_ALIAS			= RenderingHints.VALUE_ANTIALIAS_OFF;


//	DIALOG CONSTANTS
	/**
	 * Offsets are used to set/interpret table values in the Action Grid.
	 * Next value should be 32, then 64, etc.  STRIP_ALL must be updated
	 * whenever more offsets are added.
	 */
	public static final int SPEAK_OFFSET				= 1;
	public static final int THINK_OFFSET				= 2;
	public static final int EMOTE_OFFSET				= 4;
	public static final int HAS_AUDIO_OFFSET			= 8;
	public static final int EMOTION_DIVIDER				= 16;
	public static final int STRIP_ALL_OFFSET			= 15;
	 

	/**
	 * Emotion indices. 
	 */
	public static final int EMOTION_NONE				= 0;
	public static final int EMOTION_HAPPY				= 1; 
	public static final int EMOTION_SAD					= 2;
	public static final int EMOTION_EXCITED				= 3;
	public static final int EMOTION_ANGRY				= 4;
	public static final int EMOTION_CONFUSED			= 5;
	public static final int EMOTION_LOVE				= 6;
	public static final int EMOTION_SURPRISED			= 7;
	public static final int EMOTION_FLOORED				= 8;
	public static final int EMOTION_STARTLED			= 9;
	public static final int EMOTION_EMBARRASSED			= 10;
	public static final int EMOTION_SARCASTIC			= 11;
	public static final int EMOTION_SLEEPY				= 12;
	public static final int EMOTION_STRESSED			= 13;
	public static final int EMOTION_AFRAID				= 14;
	public static final int EMOTION_HOT					= 15;
	public static final int EMOTION_COLD				= 16;
	public static final int EMOTION_NAUGHTY				= 17;
	public static final int EMOTION_NERVOUS				= 18;
	public static final int EMOTION_LAUGH				= 19;
	public static final int EMOTION_WINK				= 20;
	public static final int EMOTION_INNOCENT			= 21;

	
// ACTION GRID CONSTANTS
	public static final int COLUMN_WIDTH				= 100;
	public static final int NUM_BLANK_COLS				= 25;
	public static final int NUM_BLANK_ROWS				= 8;
	public static final int ROW_HEIGHT					= 105;
	public final static int CHAR_THUMB_WIDTH 			= 60;
	public final static int CHAR_THUMB_HEIGHT 			= 80;
	public final static int DIALOG_HEIGHT				= 250;
	

}