/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Data;

import java.awt.*;
import java.io.*;

import edu.uiuc.orchid.clover.Constants;

/**
 * LocalCharacter - Defines the LocalCharacter data type.
 * A character will have a unique LocalCharacter for every
 * scene it appears in.
 * 
 * @author Terry Bradley
 * @version 04.7.29
 * @date 10/18/03
 *
 */

public class LocalCharacter implements Serializable{

	// Variables
	protected Point location;	// Character's location within a scene
	protected Dimension size;	// Character's scale factor
	protected Integer layer;	// Character's layer
								// NOT CURRENTLY USED.
	protected int imageIndex;	// Character's image
								// NOT CURRENTLY USED
	
	static final int DEFAULT_X = 0;
	static final int DEFAULT_Y = 0;
	
	// Prevent InvalidClassExceptions by hardcoding a version id.
	//  - "cloverLC"
	static final long serialVersionUID = 3121522518123l;

	/**
	 * Constructor - Defines a localCharacter at (0,0) with default
	 * size.
	 *
	 */
	public LocalCharacter() {
		location = new Point(DEFAULT_X,DEFAULT_Y);
		size = new Dimension(Constants.CLABEL_DEFAULT_WIDTH,
			Constants.CLABEL_DEFAULT_HEIGHT);
		imageIndex = 0;
	}
	
	/**
	 * Constructor - Defines a localCharacter at (x,y) with size
	 * of width by height.
	 *
	 * @param x Starting location
	 * @param y Starting location
	 * @param width Starting size
	 * @param height Starting size
	 */
	public LocalCharacter(int x, int y, int width, int height) {
		location = new Point(x,y);
		size = new Dimension(width, height);
		imageIndex = 0;
	}
	
	/**
	 * Get character's location on scene
	 * 
	 * @return Location
	 */
	public Point getLocation() {
		return location;
	}
	
	/**
	 * Get character's size in scene
	 * 
	 * @return Size
	 */
	public Dimension getSize() {
		return size;
	}

	/**
	 * Get character's image index.  NOT CURRENTLY USED.
	 * 
	 * @return Image index
	 */
	public int getImageIndex() {
		return imageIndex;
	}

	/** 
	 * Set character's location
	 * 
	 * @param d Location
	 */
	public void setLocation(Point d) {
		location = d;
	}
	
	/** 
	 * Set character's location
	 * 
	 * @param x Location
	 * @param y Location
	 */
	public void setLocation(int x, int y) {
		location = new Point(x,y);
	}

	/** 
	 * Set character's size
	 * 
	 * @param s Size
	 */
	public void setSize(Dimension s) {
		size = s;
	}
	
	/** 
	 * Set character's image index
	 * 
	 * @param i Image index
	 */
	public void setImageIndex(int i) {
		imageIndex = i;
	}
	
	/**
	 * Serialization methods.  defaultW/RObject call is likely all
	 * that is required.
	 *
	 */
	private void writeObject(ObjectOutputStream oos) throws IOException {
		oos.defaultWriteObject();
		oos.writeObject(location);
		oos.writeObject(size);
		oos.writeInt(imageIndex);
	}
	
	/**
	 * Serialization methods.  defaultW/RObject call is likely all
	 * that is required.
	 *
	 */
	private void readObject(ObjectInputStream ois) throws IOException  {
		try {
			ois.defaultReadObject();
			location	= (Point)ois.readObject();
			size		= (Dimension)ois.readObject();
			imageIndex	= ois.readInt();
		} catch (ClassNotFoundException ce){
			System.err.println(ce);
		}		
	}

}
