/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.ActionGrid;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.Tabs.ViewAndEditUI;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.Actions.*;

/**
 * DialogTableEditor:
 *  Implements the editor for the dialog table.
 *  Used when adding cells to main table.  Not sure why, though,
 *  since the cells are hardcoded as non-editable.
 * 
 * @author Terry Bradley
 * @version 04.7.31
 *
 */
public class DialogTableEditor extends AbstractCellEditor 
		implements TableCellEditor, MouseListener
{
	public static final int CLICK_COUNT_TO_EDIT = 1;
	public static final Color EDITING_BACKGROUND = Color.red;

	protected ActionButton m_button;
	protected CloverWindow m_owner;
	protected boolean m_isEditing = false;
	protected int row = 0;
	protected int col = 0;

	private static final ImageIcon SPEAK_ICON = 
		Clover.createImageIcon2(Clover.SPEAK_ICON_LOC);
	private static final ImageIcon THINK_ICON = 
		Clover.createImageIcon2(Clover.THINK_ICON_LOC);
	
	/**
	 * Constructor
	 * 
	 * @param parent
	 */
	public DialogTableEditor(CloverWindow parent){
		
		m_owner = parent;
		
		m_button = new ActionButton(0, null,
			0,null,
			"", false, parent);
	}
	
	
	public Component getTableCellEditorComponent(JTable table,
		Object value, boolean isSelected, int nRow, int nCol)
	{
		final Object valu = value;
		final JTable tab = table;

		int m_tab = m_owner.getSelectedTab();
		row = nRow;
		col = nCol;
		
		m_button.m_row = nRow;
		m_button.m_col = nCol;

		if( valu instanceof Integer ) {
			boolean empty = true;
			int val = ((Integer)valu).intValue();
			int gid = ((ViewAndEditUI)m_owner.getInterface(m_tab)).
				de.getGIDByRow(row);
			if( (val & Constants.SPEAK_OFFSET) != 0 ){
				if( gid > -1 ){
					Speak ca = (Speak)((ViewAndEditUI)m_owner.getInterface(m_tab)).
							getCurrentScene().getPrimaryAction(new Long(gid), col);
					String text = "<html>";
					
					if(ca != null)
						text += (ca.getActionText().length() > 20) ? 
								 ca.getActionText().substring(0,20)+"..." :
								 ca.getActionText();

					m_button.setText(text);
					m_button.setPrimaryIcon(SPEAK_ICON, 
						Constants.SPEAK_OFFSET);
				}

				empty = false;
			}
			if( (val & Constants.THINK_OFFSET) != 0 ) {
				if( gid > -1 ){
					Think ca = (Think)((ViewAndEditUI)m_owner.getInterface(m_tab)).
							getCurrentScene().getPrimaryAction(new Long(gid), col);
					String text = "<html>";
					text += (ca.getActionText().length() > 20) ? 
								ca.getActionText().substring(0,20)+"..." :
								ca.getActionText();

					m_button.setText(text);
					m_button.setPrimaryIcon(THINK_ICON, 
						Constants.THINK_OFFSET);
				}
				empty = false;
			}
			if(empty) {
				m_button.setText("");
				m_button.setPrimaryIcon(null, 0);
			}

			boolean addSec = false;
			if( (val & Constants.EMOTE_OFFSET) != 0 ){
				Emoticon e = Emote.getEmoticon(val / Constants.EMOTION_DIVIDER);
				m_button.setSecondaryIcon(e.m_image, Constants.EMOTE_OFFSET + 
					Constants.EMOTION_DIVIDER * e.m_index);

				addSec = true;
			}
			else if (!empty)
				m_button.setSecondaryIcon(Emote.EMOTICONS[0].m_image, 
					Constants.EMOTE_OFFSET);
			else
				m_button.setSecondaryIcon(null, 0);

		}

		return m_button;	
	}
	
	public Object getCellEditorValue(){
		int value = 0;

		value = m_button.getAction();
		return new Integer(value);
	}
	
	
	public boolean isCellEditable(EventObject anEvent){
		if( anEvent instanceof MouseEvent ) {
			int click = ((MouseEvent)anEvent).getClickCount();
			if( click == CLICK_COUNT_TO_EDIT ) {
					return true;
			}
		}
		return m_isEditing;
	}
	
	public void mouseReleased(MouseEvent e) {
			m_isEditing = false;
			fireEditingStopped();
	}
	public void mousePressed(MouseEvent e) {
		m_isEditing = true;
	}
	public void mouseClicked(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}

}