/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.CloverList;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;

import edu.uiuc.orchid.clover.Interface.Tabs.*;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Interface.*;
import edu.uiuc.orchid.clover.Interface.ActionGrid.ActionButton;
import edu.uiuc.orchid.clover.DrawingTools.BackgroundSurface;

/**
 * CharacterDialogButton
 * Very similar to CharacterButton - Used in the Gallery Dialog (found
 * in Scene editor), so some extra steps are required.
 * 
 * @author Terry Bradley
 * @version 04.8.01
 *
 */
public class CharacterDialogButton extends CloverListButton {
	
	protected CloverWindow m_frame;		// CloverWindow parent
	protected JDialog m_dialog;			// Containing dialog
	protected JLabel m_dialogLabel;		// Drag and drop label for dialog
	protected JLabel m_label;			// Drag and drop label for main frame
	protected Rectangle m_labelSize;	// Drag and drop dimensions
	
	/**
	 * Constructor - Similar to CloverListButton constructor, with the
	 * addition of two parents.
	 * 
	 * @param img Button icon
	 * @param name Button name
	 * @param id Button ID
	 * @param par CloverList parent
	 * @param normal Normal status
	 * @param frm CloverWindow parent
	 * @param dial Dialog parent
	 */
	public CharacterDialogButton(ImageIcon img, String name, 
								int id, CloverList par, boolean normal,
								CloverWindow frm, JDialog dial)
	{
		super(img, name, id, par, normal);
		m_frame = frm;
		m_dialog = dial;
		
		// Setup drag and drop labels
		m_dialogLabel = new JLabel(img);
		m_dialogLabel.setVisible(false);
		m_dialogLabel.setOpaque(true);
		m_dialogLabel.setBorder(new BevelBorder(BevelBorder.RAISED));
		m_dialogLabel.setBackground(Constants.COLOR_LIST);
		((DragGlassPane)m_dialog.getGlassPane()).add(m_dialogLabel);
		
		m_label = new JLabel(img);
		m_label.setDisabledIcon(this.getDisabledIcon());
		m_label.setVisible(false);
		m_label.setOpaque(true);
		m_label.setBackground(Constants.COLOR_LIST);
		m_label.setBorder(new BevelBorder(BevelBorder.RAISED));
		m_parent.addLabelToGlassPane(m_label);
	
		m_labelSize = new Rectangle(img.getIconWidth()+LABEL_GAP, 
			img.getIconHeight()+LABEL_GAP);	
		
		addMouseListener(createMouseAdapter());
		addMouseMotionListener(createMouseMotionAdapter());
	}
	
	/**
	 * MouseAdapter detects drag and drop events.  On drag, icons are
	 * set for both the dialog and the CloverWindow.  On drops, only
	 * BackgroundSurface is a valid target.  Dropping here adds the
	 * dropped item to the scene.
	 *  
	 * @return
	 */
	private MouseAdapter createMouseAdapter(){
		MouseAdapter ma = new MouseAdapter(){
			public void mousePressed(MouseEvent e){
				startPoint = e.getPoint();
				m_label.setIcon(CharacterDialogButton.this.getIcon());
				
				
				DragGlassPane glassPane = 
								(DragGlassPane)m_frame.getGlassPane();
				DragGlassPane dialogPane = 
								(DragGlassPane)m_dialog.getGlassPane();
								
				Point glassPoint = SwingUtilities.convertPoint(
											CharacterDialogButton.this,
											e.getPoint(),
											glassPane);
				Point dialogPoint = SwingUtilities.convertPoint(
											CharacterDialogButton.this,
											e.getPoint(),
											dialogPane);
											
				// Center cursor in label
				glassPoint.x -= m_labelSize.width / 2;
				glassPoint.y -= m_labelSize.height / 2;
				
				dialogPoint.x -= m_labelSize.width / 2;
				dialogPoint.y -= m_labelSize.height / 2;
				
				m_label.setLocation(glassPoint);
				m_label.setBounds(glassPoint.x, glassPoint.y, 
					m_labelSize.width, m_labelSize.height);
				glassPane.setPoint( glassPoint);
				
				m_dialogLabel.setLocation(dialogPoint);
				m_dialogLabel.setBounds(dialogPoint.x, dialogPoint.y, 
					m_labelSize.width, m_labelSize.height);
				dialogPane.setPoint( dialogPoint);
				
				m_label.setVisible(true);
				glassPane.setVisible(true);
				
				m_dialogLabel.setVisible(true);
				dialogPane.setVisible(true);
				
				m_frame.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				m_dialog.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
				m_dragging = true;
				m_dialog.setResizable(false);
			}
			public void mouseReleased(MouseEvent e){
				CharacterDialogButton jb = (CharacterDialogButton)e.getSource();
				Component glassPane = m_frame.getGlassPane();
				DragGlassPane dialogPane = 
								(DragGlassPane)m_dialog.getGlassPane();
				
				if( m_dragging ){
					Point glassPoint = SwingUtilities.convertPoint(
												jb,
												e.getPoint(),
												glassPane);
												
					Point dialogPoint = SwingUtilities.convertPoint(
												CharacterDialogButton.this,
												e.getPoint(),
												dialogPane);
					Component component =
								SwingUtilities.getDeepestComponentAt(
											m_frame.getContentPane(),
											glassPoint.x,
											glassPoint.y);
											
					Component dialogComponent =
								SwingUtilities.getDeepestComponentAt(
											m_dialog.getContentPane(),
											dialogPoint.x,
											dialogPoint.y);
					
					if( component != null &&
						component instanceof BackgroundSurface &&
						dialogComponent == null )
					{
						// Add character instance at e.getPoint
						// If character is in scene, VaE should beep
						// and/or highlight character.
						Point surfacePoint = SwingUtilities.convertPoint(
															jb,
															e.getPoint(),
															component);
						surfacePoint.x = Math.max(0, 
									surfacePoint.x - m_labelSize.width / 2);
						surfacePoint.y = Math.max(0,
									surfacePoint.y - m_labelSize.height / 2);
						
						int tab = m_frame.getSelectedTab();
						if(tab == Constants.VIEW_AND_EDIT_INDEX)
							((ViewAndEditUI)m_frame.getInterface(tab)).
								addExistingCharacter(m_id, surfacePoint);
						else if(tab == Constants.RESPONSE_INDEX)
							((ResponseUI)m_frame.getInterface(tab)).
								addExistingCharacter(m_id, surfacePoint);
					}

				} // was dragging
				
				m_dragging = false;
				m_dialog.setResizable(true);
				m_frame.setCursor(Cursor.getDefaultCursor());
				m_dialog.setCursor(Cursor.getDefaultCursor());
				glassPane.setVisible(false);
				m_label.setVisible(false);
				
				dialogPane.setVisible(false);
				m_dialogLabel.setVisible(false);
			}
			public void mouseClicked(MouseEvent e){}
			public void mouseExited(MouseEvent e){
				setCursor(Cursor.getDefaultCursor());
			}
			public void mouseEntered(MouseEvent e){
				setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			}
		};
		return ma;
	}
	
	/**
	 * Update drag and drop labels while dragging.
	 * 
	 * @return
	 */
	private MouseMotionAdapter createMouseMotionAdapter(){
		MouseMotionAdapter mma = new MouseMotionAdapter(){
			public void mouseDragged(MouseEvent e){
				// Pick up and display label on glassPane
				DragGlassPane glassPane = 
									(DragGlassPane)m_frame.getGlassPane();
				DragGlassPane dialogPane = 
								(DragGlassPane)m_dialog.getGlassPane();
				CharacterDialogButton jb = (CharacterDialogButton)e.getSource();
				
				Point glassPoint = SwingUtilities.convertPoint(
											jb,
											e.getPoint(),
											glassPane);
											
				Point dialogPoint = SwingUtilities.convertPoint(
											CharacterDialogButton.this,
											e.getPoint(),
											dialogPane);
											
				// Center cursor in label
				glassPoint.x -= m_labelSize.width / 2;
				glassPoint.y -= m_labelSize.height / 2;
				
				dialogPoint.x -= m_labelSize.width / 2;
				dialogPoint.y -= m_labelSize.height / 2;
				
				m_label.setLocation(glassPoint);
				m_label.setBounds(glassPoint.x, glassPoint.y, 
					m_labelSize.width, m_labelSize.height);
				glassPane.setPoint( glassPoint);
				
				m_dialogLabel.setLocation(dialogPoint);
				m_dialogLabel.setBounds(dialogPoint.x, dialogPoint.y, 
					m_labelSize.width, m_labelSize.height);
				dialogPane.setPoint( dialogPoint);				
				
				Component component =
							SwingUtilities.getDeepestComponentAt(
										m_frame.getContentPane(),
										glassPoint.x,
										glassPoint.y);
				
				Component dialogComponent =
											SwingUtilities.getDeepestComponentAt(
														m_dialog.getContentPane(),
														dialogPoint.x,
														dialogPoint.y);
													
				if( component instanceof BackgroundSurface && 
					dialogComponent == null )
				
					m_label.setIcon(CharacterDialogButton.this.getIcon());		
				
				else
					m_label.setIcon(ActionButton.BIG_X);
				

			}
		};
		return mma;
	}
	
}