/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Tabs;

import java.awt.*;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Set;
import javax.swing.ImageIcon;
import javax.swing.JLayeredPane;

import edu.uiuc.orchid.clover.Interface.CharacterLabel;
import edu.uiuc.orchid.clover.Interface.Windows.CloverWindow;
import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Data.LocalCharacter;
import edu.uiuc.orchid.clover.Data.Response;
import edu.uiuc.orchid.clover.Data.Scene;
import edu.uiuc.orchid.clover.Data.Story;

/**
 * ResponseUI - Scene Editor implementation used for creating/editing
 * responses to the main story.  Requires only a handful of changes
 * from the regular Scene Editor.
 * 
 * @author Terry J Bradley
 * @version 04.8.16
 *
 */
public class ResponseUI extends ViewAndEditUI {
	
	/**
	 * Constructor - Same as super, with the drawing surface altered to
	 * currently identify itself as within the response and not scene editor
	 * tab.
	 * 
	 * @param win CloverWindow parent
	 * @param dss Size
	 * @param name Response title
	 */
	public ResponseUI(CloverWindow win, Dimension dss, String name){
		super(win, dss, name);
		ds.setTab(Constants.RESPONSE_INDEX);
	}
	
	/**
	 * Overwritten function resets the drawing surface and
	 * restores its knowledge of this tab.
	 */
	public void resetDrawingSurface(Dimension d){
		super.resetDrawingSurface(d);
		ds.setTab(Constants.RESPONSE_INDEX);
	}
	
	/**
	 * Overwritten openStory function is called whenever a story
	 * is opened.  For a response, elements should be reinitialized, but
	 * nothing has to be loaded until the tab is opened.
	 * 
	 */
	public void openStory(Dimension size){
		resetDrawingSurface(size);
		
		populateVAE();
		
		m_scene = null;
	}
	
	/**
	 * Overwritten function consists entirely of resetting the drawing
	 * surface.
	 * 
	 */
	public void updateVaEUI(Dimension size){
		resetDrawingSurface(size);
	}
	
	/**
	 * Overwritten function handles the character labels.
	 */
	public void populateVAE() {
		Story sty = m_parent.getStory();
		
		m_CLabels.clear();
		ds.resetSurface();
		
		if( sty == null )
			return;
		
		for( int j=1; j<sty.getResponseCount(); j++) {
			for( int i=0; i<sty.getSceneCount(j); i++ ) {
				ArrayList clabels = new ArrayList();
				Scene scene = sty.getScene(i, j);
				Set s = scene.getCharacterIDs();
				for( Iterator k = s.iterator(); k.hasNext(); ){
					int gid = ((Integer)k.next()).intValue();
					LocalCharacter lc = scene.getCharacter(gid);
					
					ImageIcon img = new ImageIcon(
						sty.getCharacter(gid).openImage().
							getScaledInstance(lc.getSize().width,
											  lc.getSize().height,
											  Image.SCALE_AREA_AVERAGING));
					CharacterLabel cl = new CharacterLabel(
						img, gid, this);
											
					cl.setBounds(lc.getLocation().x, lc.getLocation().y,
								 lc.getSize().width, lc.getSize().height);
					clabels.add(cl);
					
					cl.setVisible(false);
					ds.add(cl, JLayeredPane.DRAG_LAYER);
				}
				m_CLabels.put(new Integer(scene.getSceneID()), clabels);	// was m_scene
			}
		}
	}
	
	/**
	 * Updates the gallery of scene thumbnails for the currently open response.
	 *
	 */
	public void updateGallery(){
		updateGallery(m_responseIndex);
	}

	/**
	 * Loads the given response.  Called when the Response tab is opened
	 * and a response is chosen.
	 * 
	 * @param index Response index
	 */
	public void loadResponse(int index){
		// Set m_responseIndex
		m_responseIndex = index;
		
		// Get response
		Response r = (Response)getStory().getResponse(index);
		
		// Add the first scene if needed
		if(r.getSceneCount() < 1)
			addScene();
		
		updateGallery(index);
			
		// Construct gallery
		m_sceneList.constructList(getStory().getSceneOrder(index));
		
		// Load first scene
		openSceneWithID(r.getSceneAtPosition(0));
	}

}