/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Interface.Windows;

import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.awt.event.*;

import edu.uiuc.orchid.clover.*;

/**
 * CloverSplash
 * Displays an undecorated window - i.e. a Splash screen (also,
 * with addition of an OK button, suitable for Help->About)
 * 
 * @author Terry Bradley
 * @version 04.8.17
 *
 */
public class CloverSplash extends JFrame {
	
	JTextArea m_text;		// Dynamic "Loading..." text box
	
	/**
	 * Creates the splash screen.  Only the m_text box changes as
	 * the program loads.
	 * 
	 * @param ver Version info
	 * @param img Icon for title bar
	 */
	public CloverSplash(String ver, Image img){
		super();
		setUndecorated(true);
		getRootPane().setWindowDecorationStyle(JRootPane.FRAME);
		setIconImage(img);
		setTitle("Clover - Loading...");
		setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		
		// Clover Icon
		JLabel clover = new JLabel(Clover.createImageIcon2
						(Clover.CLOVER_SPLASH_ICON_LOC));
		clover.setAlignmentX(Component.CENTER_ALIGNMENT);
		
		// Title panel
		JLabel title = new JLabel("Clover");
		title.setFont(new Font(title.getFont().getFamily(), Font.BOLD, 24));
		title.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		JLabel version = new JLabel("version "+ver);
		version.setFont(new Font(version.getFont().getFamily(), Font.BOLD, 10));
		version.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		JPanel titlePane = new JPanel();
		titlePane.setLayout(new BoxLayout(titlePane, BoxLayout.Y_AXIS));
		titlePane.setBackground(new Color(230,230,230));
		titlePane.add(title);
		titlePane.add(version);
		
		// Copyright label
		JLabel copyright = new JLabel("<html>&copy;&nbsp; Copyright UIUC 2004." +
									  "&nbsp;  All rights reserved.");
		
		// Credits pane
		JLabel author = new JLabel("Author: ");
		author.setAlignmentX(Component.LEFT_ALIGNMENT);
		JLabel terry = new JLabel("Terry Bradley");
		terry.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		JLabel advisors = new JLabel("Advisors: ");
		advisors.setAlignmentX(Component.LEFT_ALIGNMENT);
		JLabel brian = new JLabel("Brian Bailey");
		brian.setAlignmentX(Component.LEFT_ALIGNMENT);
		JLabel sharon = new JLabel("Sharon Tettegah");
		sharon.setAlignmentX(Component.LEFT_ALIGNMENT);
		
		JPanel advisorPane = new JPanel();
		advisorPane.setLayout(new GridBagLayout());
		advisorPane.setBackground(new Color(230,230,230));
		advisorPane.setAlignmentX(Component.LEFT_ALIGNMENT);
		GridBagConstraints c = new GridBagConstraints();
		c.insets = new Insets(0,3,3,3); 
		c.anchor = GridBagConstraints.WEST;
		advisorPane.add(author, c);
		c.gridx = 1;
		advisorPane.add(terry, c);
		c.gridx = 0;
		c.gridy = 1;
		c.insets = new Insets(0,3,0,3);
		advisorPane.add(advisors, c);
		c.gridx = 1;
		advisorPane.add(brian, c);
		c.gridy = 2;
		advisorPane.add(sharon, c);
		
		// Status area
		m_text = new JTextArea(2, 50);
		m_text.setEditable(false);
		m_text.setFont(new Font("Arial", Font.PLAIN, 10));
		m_text.setBorder(new CompoundBorder(
			new LineBorder(Color.black),
			new EmptyBorder(0,3,0,0)		));
			
		// Orchid research group
		JLabel orchid = new JLabel("<html>Orchid Research Group<br>" +
			"<a href=\"http://orchid.cs.uiuc.edu/\">http://orchid.cs.uiuc.edu</a>");
		MouseAdapter ma = new MouseAdapter(){
			public void mouseClicked(MouseEvent e){
				if(Clover.WIN_LOOK_AND_FEEL){
					try{
						Runtime.getRuntime().exec("cmd.exe /c start iexplore http://orchid.cs.uiuc.edu");
						//BasicService  bs = ( BasicService )
		   				//	ServiceManager.lookup ( "javax.jnlp.BasicService" );
						//bs.showDocument(new java.net.URL("http://orchid.cs.uiuc.edu"));
					}
					catch(Exception ex){
						System.err.println("Error launching browser");
					}
				}
			}
			public void mouseEntered(MouseEvent e){
				((JLabel)e.getSource()).setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
			}
			public void mouseExited(MouseEvent e){
				((JLabel)e.getSource()).setCursor(Cursor.getDefaultCursor());
			}
		};
		orchid.addMouseListener(ma);
		
		// Main Container
		JPanel pane = new JPanel();
		pane.setLayout(new GridBagLayout());
		c = new GridBagConstraints();
		c.gridheight = 5;
		pane.add(clover, c);
		
		c.gridx = 1;
		c.gridheight = 1;
		c.insets = new Insets(5,5,15,5);
		pane.add(titlePane, c);
		
		c.gridy = 1;
		pane.add(copyright, c);
		
		c.gridy = 2;
		pane.add(advisorPane, c);
		
		c.gridy = 3;
		pane.add(orchid, c);
		
		c.gridx = 0;
		c.gridy = 5;
		c.gridwidth = 2;
		pane.add(m_text, c);
		pane.setBorder(new LineBorder(Color.black));
		pane.setBackground(new Color(230,230,230));
		
		getContentPane().add(pane);
		
		pack();
		setLocationRelativeTo(null);
		setVisible(true);
	}
	
	/**
	 * Returns the status text from m_text.
	 * 
	 * @return Status text
	 */
	public String getStatusText() {
		return m_text.getText();
	}

	/**
	 * Sets the status text in m_text.  Only two lines are saved, so
	 * the first line must be discarded first.
	 * 
	 * @param status New status text
	 */
	public void setStatusText(String status) {
		// Get old text, discard top line
		String text = removeFirstLine(m_text.getText());
		m_text.setText(text+"[done]\n"+status);
	}
	
	/**
	 * Helper function removes the first line from m_text.
	 * 
	 * @param in Initial text
	 * @return Remaining text
	 */
	private String removeFirstLine(String in){
		String[] strings = in.split("\n", 2);
		if(strings.length == 2)
			return strings[1];
		else
			return in;
	}

}