/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

import java.awt.Point;
import java.awt.Dimension;
import javax.swing.ImageIcon;

/**
 * Bubble Class
 * Simple dataclass, stores Icon and crucial info about different
 * styles of bubbles.
 * 
 * @author Terry Bradley
 * @version 04.8.19
 *
 */
public class Bubble {
	
	private ImageIcon m_img;		// Bubble appearance
	private Dimension m_bubbleDim;	// Interior bubble dimensions
	private Dimension m_totalDim;	// Exterior bubble dimensions
	private Point m_location;		// Point where callout ends
	private int maxLines;			// Number of lines bubble can hold
	
	/**
	 * Constructor - Assigns values to all the private variables.
	 * 
	 * @param img Bubble appearance
	 * @param loc Point where callout ends
	 * @param bubble Interior bubble dimensions
	 * @param total Exterior bubble dimensions
	 * @param lines Number of lines bubble can hold
	 */
	public Bubble(ImageIcon img, Point loc, 
				  Dimension bubble, Dimension total,
				  int lines)
	{
		m_img = img;
		m_location = loc;
		maxLines = lines;
		m_bubbleDim = bubble;
		m_totalDim = total;
	}
	
	/**
	 * Returns bubble's appearance
	 * 
	 * @return Bubble's appearance
	 */
	public ImageIcon getBubble(){
		return m_img;
	}

	/**
	 * Returns bubble's callout point
	 * 
	 * @return Bubble's callout point
	 */
	public Point getPoint(){
		return m_location;		
	}
	
	/**
	 * Returns bubble's maximum number of lines
	 * 
	 * @return Number of lines bubble can hold
	 */
	public int getMaxLines(){
		return maxLines;
	}
	
	/**
	 * Returns bubble's interior width
	 * 
	 * @return Bubble's interior width
	 */
	public int getBubbleWidth(){
		return m_bubbleDim.width;
	}
	
	/**
	 * Returns bubble's interior height
	 * 
	 * @return Bubble's interior height
	 */
	public int getBubbleHeight(){
		return m_bubbleDim.height;
	}
	
	/**
	 * Returns bubble's exterior width
	 * 
	 * @return Bubble's exterior width
	 */
	public int getTotalWidth(){
		return m_totalDim.width;
	}
	
	/**
	 * Returns bubble's exterior height
	 * 
	 * @return Bubble's exterior height
	 */
	public int getTotalHeight(){
		return m_totalDim.height;
	}
}