/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

import javax.swing.JLabel;

import edu.uiuc.orchid.clover.*;
import edu.uiuc.orchid.clover.Actions.*;

/**
 * EmotionBubble - Simple bubble for displaying static visual emotions
 * (or animated gifs).
 *  
 * @author Terry J Bradley
 * @version 04.8.19
 *
 */
public class EmotionBubble extends JLabel {
	
	/**
	 * Array of all supported emotions. (Future emotions commented out)
	 */
	public static Emotion[] EMOTIONS = {
		//new Emotion(Clover.EMOTION_NONE_LOC, "None", EMOTION_NONE),
		new Emotion(Clover.createImageIcon2(Clover.EMOTION_HAPPY_ACTIVE_LOC),
					Clover.createImageIcon2(Clover.EMOTION_HAPPY_STATIC_LOC),
					Clover.EMOTION_HAPPY_AUDIO_LOC,
					Constants.EMOTION_HAPPY),
		new Emotion(Clover.createImageIcon2(Clover.EMOTION_SAD_ACTIVE_LOC),
					Clover.createImageIcon2(Clover.EMOTION_SAD_STATIC_LOC),
					Clover.EMOTION_SAD_AUDIO_LOC,
					Constants.EMOTION_SAD),
		//new Emoticon(EMOTION_EXCITED_LOC, "Excited", EMOTION_EXCITED),
		new Emotion(Clover.createImageIcon2(Clover.EMOTION_ANGRY_ACTIVE_LOC),
					Clover.createImageIcon2(Clover.EMOTION_ANGRY_STATIC_LOC),
					Clover.EMOTION_ANGRY_AUDIO_LOC,
					Constants.EMOTION_ANGRY),
		//new Emoticon(EMOTION_INNOCENT_LOC, "Innocent", EMOTION_INNOCENT),
		new Emotion(Clover.createImageIcon2(Clover.EMOTION_CONFUSED_ACTIVE_LOC),
					Clover.createImageIcon2(Clover.EMOTION_CONFUSED_STATIC_LOC),
					Clover.EMOTION_CONFUSED_AUDIO_LOC,
					Constants.EMOTION_CONFUSED),
		new Emotion(Clover.createImageIcon2(Clover.EMOTION_LOVE_ACTIVE_LOC),
					Clover.createImageIcon2(Clover.EMOTION_LOVE_STATIC_LOC),
					Clover.EMOTION_LOVE_AUDIO_LOC,
					Constants.EMOTION_LOVE),
		new Emotion(Clover.createImageIcon2(Clover.EMOTION_SURPRISED_ACTIVE_LOC),
					Clover.createImageIcon2(Clover.EMOTION_SURPRISED_STATIC_LOC),
					Clover.EMOTION_SURPRISED_AUDIO_LOC,
					Constants.EMOTION_SURPRISED),
		//new Emoticon(EMOTION_FLOORED_LOC, "Floored", EMOTION_FLOORED),
		//new Emoticon(EMOTION_STARTLED_LOC, "Startled", EMOTION_STARTLED),
		new Emotion(Clover.createImageIcon2(Clover.EMOTION_EMBARRASSED_ACTIVE_LOC),
					Clover.createImageIcon2(Clover.EMOTION_EMBARRASSED_STATIC_LOC),
					Clover.EMOTION_EMBARRASSED_AUDIO_LOC,
					Constants.EMOTION_EMBARRASSED),
		//new Emoticon(EMOTION_SARCASTIC_LOC, "Sarcastic", EMOTION_SARCASTIC),
		//new Emoticon(EMOTION_SLEEPY_LOC, "Sleepy", EMOTION_SLEEPY),
		//new Emoticon(EMOTION_STRESSED_LOC, "Stressed", EMOTION_STRESSED),
		//new Emoticon(EMOTION_HOT_LOC, "Hot", EMOTION_HOT),
		//new Emoticon(EMOTION_COLD_LOC, "Cold", EMOTION_COLD),
		//new Emoticon(EMOTION_NAUGHTY_LOC, "Naughty", EMOTION_NAUGHTY),
		//new Emoticon(EMOTION_NERVOUS_LOC, "Nervous", EMOTION_NERVOUS),
		//new Emoticon(EMOTION_LAUGH_LOC, "Laugh", EMOTION_LAUGH),
		new Emotion(Clover.createImageIcon2(Clover.EMOTION_WINK_ACTIVE_LOC),
					Clover.createImageIcon2(Clover.EMOTION_WINK_STATIC_LOC),
					Clover.EMOTION_WINK_AUDIO_LOC,
					Constants.EMOTION_WINK)
	};
	
	/**
	 * Constructor - Ignores appearance and only sets a default size
	 * of 131x131.
	 *
	 */
	public EmotionBubble(){
		super();
		setSize(131,131);
	}

}