/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

import edu.uiuc.orchid.clover.Data.*;

/**
 * ProcessStory - Story processing thread that produces scenes for
 * ProcessScene to consume.  Once the last scene is added, this thread
 * sets stop, which propogates to halt the other processing threads.
 * 
 * @author Terry J Bradley
 * @version 04.8.19
 *
 */
public class ProcessStory extends Thread {

	private SceneSynch sceneSynch;	// Shared scene synch instance
	private Story m_story;			// Story
	private Integer[] m_scenes;		// Array of scenes to play
	private Playable play;			// Playable interface

	/**
	 * Constructor - Sets the processStory variables.
	 * 
	 * @param s Shared scene synch instance
	 * @param p Playable interface
	 * @param sty Story
	 * @param scenes Array of scene IDs
	 */
	public ProcessStory(SceneSynch s, Playable p, Story sty, 
						Integer[] scenes)
	{
		sceneSynch = s;
		play = p;
		m_story = sty;
		m_scenes = scenes;
	}

	/**
	 * Linearly produces scenes for the ProcessScene thread.  Then sends
	 * empty "stop" scene to cleanup the other threads.
	 */
	public void run() {
		// get next scene
		for(int i=0; i<m_scenes.length; i++){
			sceneSynch.putScene(m_story.getScene(m_scenes[i]), false);
		}
		sceneSynch.putScene(null, true);
		play.colorCell(null, false);
	}//run
	
}