/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

import edu.uiuc.orchid.clover.Data.*;

/**
 * SceneSynch - Scene synchronization methods that able
 * scene producer/consumer models with a single element
 * buffer.
 * 
 * @author Terry J Bradley
 * @version 04.8.19
 * 
 */
public class SceneSynch{
	
	private Scene scene;			// Scene produced, not yet consumed			
	private boolean busy = false;	// True during processing

	/**
	 * Consumes the scene stored in scene.
	 * 
	 * @return Consumed scene
	 */
	public synchronized Scene getScene() {
		while (busy == false) {
			try {
				// wait for Producer to put value
				wait();
			} catch (InterruptedException e) {
			}
		}
		busy = false;
		// notify Producer that value has been retrieved
		notifyAll();
		return scene;
	}
	
	/**
	 * Stores a scene from a producer.
	 * 
	 * @param s Scene to store
	 * @param last If true, a null scene is stored, stopping the
	 * processing threads.
	 */
	public synchronized void putScene(Scene s, boolean last) {
		while (busy == true) {
			try {
				// wait for Consumer to get value
				wait();
			} catch (InterruptedException e) {
			}
		}
		if(last)
			scene = null;
		else
			scene = s;
		busy = true;
		// notify Consumer that value has been set
		notifyAll();
	} 	
	
}