/******************************************************************************\
* University of Illinois/NCSA                                                  *
* Open Source License                                                          *
*                                                                              *
* Copyright 2004, University of Illinois at Urbana-Champaign.  All rights     *
* reserved.                                                                    *
*                                                                              *
* Developed by:                                                                *
*                                                                              *
*   Terry Bradley                                                              *
*   Brian Bailey                                                               *
*   Sharon Tettegah                                                            *
*                                                                              *
*   Orchid Research Group                                                      *
*                                                                              *
*   University of Illinois at Urbana-Champaign                                 *
*                                                                              *
*   http://orchid.cs.uiuc.edu/                                                 *
*                                                                              *
* Permission is hereby granted, free of charge, to any person obtaining a      *
* copy of this software and associated documentation files (the Software),   *
* to deal with the Software without restriction, including without limitation  *
* the rights to use, copy, modify, merge, publish, distribute, sublicense,     *
* and/or sell copies of the Software, and to permit persons to whom the        *
* Software is furnished to do so, subject to the following conditions:         *
*                                                                              *
*   * Redistributions of source code must retain the above copyright notice,   *
*     this list of conditions and the following disclaimers.                   *
*                                                                              *
*   * Redistributions in binary form must reproduce the above copyright        *
*     notice, this list of conditions and the following disclaimers in the     *
*     documentation and/or other materials provided with the distribution.     *
*                                                                              *
*   * Neither the names of the Orchid Research Group, the University of        *
*     Illinois at Urbana-Champaign, nor the names of its contributors may be   *
*     used to endorse or promote products derived from this Software without   *
*     specific prior written permission.                                       *
*                                                                              *
* THE SOFTWARE IS PROVIDED AS IS, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR   *
* IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,     *
* FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL     *
* THE CONTRIBUTORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR    *
* OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
* ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER  *
* DEALINGS WITH THE SOFTWARE.                                                  *
\******************************************************************************/

package edu.uiuc.orchid.clover.Playback;

import java.awt.*;

import edu.uiuc.orchid.clover.*;

/**
 * SpeakBubble - Instance of ActionBubble used to show spoken
 * dialog.  The 8 possible bubbles of 4 different sizes are stored
 * as SPEECH_BUBBLE_0 through 3 and SPEECH_BUBBLE_LEFT_0 through 3.
 * 
 * @author Terry J Bradley
 * @version 04.8.19
 *
 */
public class SpeakBubble extends ActionBubble{
	
	/**
	 * Defines the 4 right bubbles, loading their images from
	 * the disk.  Bubble sizes are hardcoded, based on the images.
	 */
	private static final Bubble SPEECH_BUBBLE_0 =
		new Bubble(Clover.createImageIcon2(Clover.SPEECH_BUBBLE_0_LOC),
			new Point(12,72), new Dimension(90, 60), 
			new Dimension(92,72), 3);
	private static final Bubble SPEECH_BUBBLE_1 =
		new Bubble(Clover.createImageIcon2(Clover.SPEECH_BUBBLE_1_LOC),
			new Point(22,114), new Dimension(132, 106), 
			new Dimension(134,120), 5);
	private static final Bubble SPEECH_BUBBLE_2 =
		new Bubble(Clover.createImageIcon2(Clover.SPEECH_BUBBLE_2_LOC),
			new Point(30,120), new Dimension(166,112), 
			new Dimension(168,124), 6);
	private static final Bubble SPEECH_BUBBLE_3 =
		new Bubble(Clover.createImageIcon2(Clover.SPEECH_BUBBLE_3_LOC),
			new Point(39,151), new Dimension(206, 144), 
			new Dimension(208,160), 8);
			
	/**
	 * Defines the 4 left bubbles, loading their images from
	 * the disk.  Bubble sizes are hardcoded, based on the images.
	 */
	private static final Bubble SPEECH_BUBBLE_LEFT_0 =
		new Bubble(Clover.createImageIcon2(Clover.SPEECH_BUBBLE_LEFT_0_LOC),
			new Point(80,72), new Dimension(90, 60), 
			new Dimension(92,72), 3);
	private static final Bubble SPEECH_BUBBLE_LEFT_1 =
		new Bubble(Clover.createImageIcon2(Clover.SPEECH_BUBBLE_LEFT_1_LOC),
			new Point(112,114), new Dimension(132, 106), 
			new Dimension(134,120), 5);
	private static final Bubble SPEECH_BUBBLE_LEFT_2 =
		new Bubble(Clover.createImageIcon2(Clover.SPEECH_BUBBLE_LEFT_2_LOC),
			new Point(138,120), new Dimension(166,112), 
			new Dimension(168,124), 6);
	private static final Bubble SPEECH_BUBBLE_LEFT_3 =
		new Bubble(Clover.createImageIcon2(Clover.SPEECH_BUBBLE_LEFT_3_LOC),
			new Point(169,151), new Dimension(206, 144), 
			new Dimension(208,160), 8);
	
	/**
	 * Sets the bubble text, using the max lines property and an
	 * appropriate bubble size.
	 */
	public void setBubbleText(String text, int start, int end){
		if(text.length() < 16) {
			super.setBubbleText(text, 0, SPEECH_BUBBLE_0.getMaxLines());
		}
		else if(text.length() < 55){
			super.setBubbleText(text, 0, SPEECH_BUBBLE_1.getMaxLines());
		}
		else if(text.length() < 76){
			super.setBubbleText(text, 0, SPEECH_BUBBLE_2.getMaxLines());
		}
		else{				// handles at most 130 char - scroll any more.
			super.setBubbleText(text, start, end);
		}
		repaint();
	}
	
	/**
	 * The bubble image used depends on the amount of text and the
	 * side the callout should appear on.
	 * 
	 * @param text Text to display
	 * @param right Side of callout
	 */
	public void setBubbleSize(String text, boolean right){
		if(text.length() < 16) {
			bubbleHeight = 60;
			setIcon((right) ? SPEECH_BUBBLE_0.getBubble() :
							  SPEECH_BUBBLE_LEFT_0.getBubble());
			setSize(92, 72);
		}
		else if(text.length() < 55){	
			bubbleHeight = 106;
			setIcon((right) ? SPEECH_BUBBLE_1.getBubble() :
				  SPEECH_BUBBLE_LEFT_1.getBubble());		
			setSize(134,120);
		}
		else if(text.length() < 76){
			bubbleHeight = 112;
			setIcon((right) ? SPEECH_BUBBLE_2.getBubble() :
				  SPEECH_BUBBLE_LEFT_2.getBubble());
			setSize(168,124);
		}
		else{				// handles at most 130 char - scroll any more.
			bubbleHeight = 144;
			setIcon((right) ? SPEECH_BUBBLE_3.getBubble() :
				  SPEECH_BUBBLE_LEFT_3.getBubble());
			setSize(208,160);
		}
		super.setBubbleText(" ", 0, 1);
	}
	
	/**
	 * Returns the point at which the callout ends.
	 * 
	 * @param length Size of text (determines bubble size)
	 * @param right Side of callout
	 * @return Callout point
	 */
	public static Point getDialogPoint(int length, boolean right){
		if(length < 16) {
			return (right) ? SPEECH_BUBBLE_0.getPoint() :
							 SPEECH_BUBBLE_LEFT_0.getPoint();
		}
		else if(length < 55){
			return (right) ? SPEECH_BUBBLE_1.getPoint() :
							 SPEECH_BUBBLE_LEFT_1.getPoint();
		}
		else if(length < 76){
			return (right) ? SPEECH_BUBBLE_2.getPoint() :
							 SPEECH_BUBBLE_LEFT_2.getPoint();
		}
		else{				// handles at most 130 char - scroll any more.
			return (right) ? SPEECH_BUBBLE_3.getPoint() :
							 SPEECH_BUBBLE_LEFT_3.getPoint();
		}
	}
	
}